#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file Component.py
# component classes

""" component widget data """

import os
import sys

from PyQt5.QtCore import (QModelIndex, Qt,
                          QFileInfo, QFile, QIODevice, QTextStream)
from PyQt5.QtWidgets import (QWidget, QGridLayout, QApplication,
                             QMenu, QFileDialog, QMessageBox)
from PyQt5.QtXml import (QDomDocument)


from .FieldDlg import FieldDlg
from .GroupDlg import GroupDlg
from .LinkDlg import LinkDlg
from .RichAttributeDlg import RichAttributeDlg
from .DataSourceDlg import DataSourceDlg
from .StrategyDlg import StrategyDlg
from .DefinitionDlg import DefinitionDlg
from .Merger import Merger, MergerDlg, IncompatibleNodeError
from .ComponentModel import ComponentModel
from .DomTools import DomTools
from .ComponentDlg import ComponentDlg

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


# Component data
class Component(object):

    # constructor
    # \brief Sets variables
    def __init__(self, parent=None):

        # directory from which components are loaded by default
        self.directory = ""
        # component view
        self.view = None
        # component id
        self.id = None
        # component name
        self.name = ""
        # component dialog
        self.dialog = None
        # component DOM document
        self.document = None
        # parent node
        self.parent = parent

        if os.path.exists(os.path.join(os.getcwd(), "components")):
            self._xmlPath = os.path.abspath(os.path.join(
                os.getcwd(), "components"))
        else:
            self._xmlPath = os.getcwd()

        if os.path.exists(os.path.join(os.getcwd(), "datasources")):
            self._dsPath = os.path.abspath(os.path.join(
                os.getcwd(), "datasources"))
        else:
            self._dsPath = os.getcwd()

        self._componentFile = None

#        # item frame
#        self.frame = None
        # component actions
        self._actions = None

        # save action
        self.externalSave = None
        # strore action
        self.externalStore = None
        # apply action
        self.externalApply = None
        # close action
        self.externalClose = None
        # datasource link action
        self.externalDSLink = None

        # item class shown in the frame
        self._tagClasses = {"field": FieldDlg,
                            "group": GroupDlg,
                            "definition": DefinitionDlg,
                            "attribute": RichAttributeDlg,
                            "link": LinkDlg,
                            "datasource": DataSourceDlg,
                            "strategy": StrategyDlg
                            }

        # current component tag
        self._currentTag = None
        self._frameLayout = None

        # if merging compited
        self._merged = False

        # merger dialog
        self._mergerdlg = None

        # merger
        self._merger = None

        # show all attribures or only the type attribute
        self._allAttributes = False

        # saved XML
        self.savedXML = None

        # tag counter
        self._tagCnt = 0

        # sub components
        self.components = []
        # sub datasources
        self.datasources = []

    # fetches $datasources and $components from xml
    # \brief populate components and datasources
    def fetchElements(self):
        dss = set()
        cps = set()
        if hasattr(self.document, "toString"):
            xml = unicode(self.document.toString(0))
            dss = set(DomTools.findElements(xml, "datasources"))
            cps = set(DomTools.findElements(xml, "components"))
        self.components = list(cps)
        self.datasources = list(dss)

    # provides attribute flag
    # \returns flag if all attributes have to be shown
    def getAttrFlag(self):
        return self._allAttributes

    # checks if not saved
    # \returns True if it is not saved
    def isDirty(self):
        string = self.get()
        return False if string == self.savedXML else True

    # provides the path of component tree for a given node
    # \param node DOM node
    # \returns path represented as a list with elements:
    #         (row number, node name)
    @classmethod
    def _getPathFromNode(cls, node):
        ancestors = [node]
        path = []

        while unicode(
            ancestors[0].parentNode().nodeName()).strip() != '#document' and \
                unicode(ancestors[0].parentNode().nodeName()).strip() != '':
            ancestors.insert(0, ancestors[0].parentNode())
        ancestors.insert(0, ancestors[0].parentNode())

        parent = None
        for child in ancestors:
            if parent:
                row = DomTools.getNodeRow(child, parent)

                if row is None:
                    path = []
                    break

                path.append((row, unicode(child.nodeName())))
            parent = child
        if not path:
            path = [(0, 'definition')]
        return path

    # provides the current component tree item
    # \returns DOM node instance
    def _getCurrentNode(self):
        index = QModelIndex()
        if self.view and self.dialog:
            index = self.view.currentIndex()
        if not index.isValid():
            return
        item = index.internalPointer()
        if not item:
            return
        return item.node

    # provides the current view index
    # \returns the current view index
    def currentIndex(self):
        index = QModelIndex()
        if self.view and self.dialog:
            try:
                index = self.view.currentIndex()
            except Exception:
                pass
        return index

    # provides the path of component tree for the current component tree item
    # \returns path represented as a list with elements:
    #          (row number, node name)
    def _getPath(self):
        index = self.currentIndex()
        pindex = index.parent()
        path = []
        if not index.isValid():
            return
        row = 1
        while pindex.isValid() and row is not None:
            child = index.internalPointer().node
            parent = pindex.internalPointer().node
            row = DomTools.getNodeRow(child, parent)
            path.insert(0, (row, unicode(child.nodeName())))
            index = pindex
            pindex = pindex.parent()

        child = index.internalPointer().node
        row = DomTools.getNodeRow(child, self.document)
        path.insert(0, (row, unicode(child.nodeName())))

        return path

    # selects and opens the last nodes of the given list in the component tree
    # \param nodes list of DOM nodes
    def _showNodes(self, nodes):
        for node in nodes:
            path = self._getPathFromNode(node)
            self._selectItem(path)

    # provides  index of the component item defined by the path
    # \param path path represented as a list with elements:
    #        (row number, node name)
    # \returns component item index
    def _getIndex(self, path):
        if not path or not self.view or not self.dialog:
            return QModelIndex()
        index = self.view.model().rootIndex
        self.view.expand(index)
        for step in path:
            index = self.view.model().index(step[0], 0, index)
            self.view.expand(index)
        return index

    # selectes item defined by path in component tree
    # \param path path represented as a list with elements:
    #        (row number, node name)
    def _selectItem(self, path):
        index = self._getIndex(path)

        if index and index.isValid():
            self.view.setCurrentIndex(index)
            self.tagClicked(index)
            self.view.expand(index)

    # provides the state of the component dialog
    # \returns tuple with (xml string, path)
    def getState(self):
        path = self._getPath()
        return (self.document.toString(0), path)

    # sets the state of the component dialog
    # \param state tuple with (xml string, path)
    def setState(self, state):
        (xml, path) = state
        try:
            self._loadFromString(xml)
        except (IOError, OSError, ValueError) as e:
            error = "Failed to load: %s" % e
            logger.warn(error)
        self._hideFrame()
        self._selectItem(path)

    # updates the component dialog
    # \brief It creates model and frame item
    def updateForm(self):
        self.dialog.ui.splitter.setStretchFactor(0, 1)
        self.dialog.ui.splitter.setStretchFactor(1, 1)

        model = ComponentModel(self.document, self._allAttributes, self.parent)
        self.view.setModel(model)
        self.connectView()

        self.dialog.ui.widget = QWidget(self.dialog)
        self._frameLayout = QGridLayout()
        self._frameLayout.addWidget(self.dialog.ui.widget)
        self.dialog.ui.frame.setLayout(self._frameLayout)

    # applies component item
    # \brief it checks if item widget exists and calls apply of the item widget
    def applyItem(self):
        if not self.view or not self.view.model() or not self.dialog \
                or not self.dialog.ui or not self.dialog.ui.widget:
            return
        if not hasattr(self.dialog.ui.widget, 'apply'):
            return
#        import gc
#        gc.collect()
        self.dialog.ui.widget.apply()

        self.view.resizeColumnToContents(0)
        self.view.resizeColumnToContents(1)
        self.fetchElements()
        return True

    # moving node up
    # \param node DOM node
    # \param parent parent node index
    # \returns the new row number if changed otherwise None
    def _moveNodeUp(self, node, parent):
        if self.view is not None and self.dialog is not None \
                and self.view.model() is not None:
            if not parent.isValid():
                return
            parentItem = parent.internalPointer()
            pnode = parentItem.node
            row = DomTools.getNodeRow(node, pnode)
            if row is not None and row != 0:
                self.view.model().removeItem(row, parent)
                self.view.model().insertItem(row - 1, node, parent)
                return row - 1

    # moving node down
    # \param node DOM node
    # \param parent parent node index
    # \returns the new row number if changed otherwise None
    def _moveNodeDown(self, node, parent):
        if self.view is not None and self.dialog is not None \
                and self.view.model() is not None:
            if not parent.isValid():
                return
            parentItem = parent.internalPointer()
            pnode = parentItem.node
            row = DomTools.getNodeRow(node, pnode)
            if row is not None and row < pnode.childNodes().count() - 1:
                self.view.model().removeItem(row, parent)
                if row < pnode.childNodes().count() - 1:
                    self.view.model().insertItem(row + 1, node, parent)
                else:
                    self.view.model().appendItem(node, parent)
                return row + 1

    # moves component item up
    # \returns the new row number if item move othewise None
    def moveUpItem(self):
        if not self.view or not self.dialog or not self.view.model():
            return
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel or not index.isValid():
            return
        node = sel.node
        parent = index.parent()
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, parent)
        row = self._moveNodeUp(node, parent)
        if row is not None:
            index = self.view.model().index(row, 0, parent)
            self.view.setCurrentIndex(index)
            self.view.model().dataChanged.emit(index, index)
            self.view.model().dataChanged.emit(parent, parent)
            return row

    # moves component item up
    # \returns the new row number if item move othewise None
    def moveDownItem(self):
        if not self.view or not self.dialog or not self.view.model():
            return
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel or not index.isValid():
            return
        node = sel.node
        parent = index.parent()
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, parent)
        row = self._moveNodeDown(node, parent)
        if row is not None:
            index = self.view.model().index(row, 0, parent)
            self.view.setCurrentIndex(index)
            self.view.model().dataChanged.emit(index, index)
            self.view.model().dataChanged.emit(parent, parent)
            return row

    # converts DOM node to XML string
    # \param component DOM node
    # \returns XML string
    @classmethod
    def _nodeToString(cls, node):
        doc = QDomDocument()
        child = doc.importNode(node, True)
        doc.appendChild(child)
        return unicode(doc.toString(0))

    # converts XML string to DOM node
    # \param xml XML string
    # \returns component DOM node
    def _stringToNode(self, xml):
        doc = QDomDocument()

        if not unicode(xml).strip():
            return
        if not doc.setContent(unicode(xml).strip()):
            raise ValueError("could not parse XML")
        if self.document and doc and doc.hasChildNodes():
            return self.document.importNode(doc.firstChild(), True)

    # pastes the component item from the clipboard into the component tree
    # \returns True on success
    def pasteItem(self):
        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget \
                or not hasattr(self.dialog.ui.widget, "subItems"):
            return

        clipboard = QApplication.clipboard()
        clipNode = self._stringToNode(clipboard.text())
        if clipNode is None:
            return

        name = unicode(clipNode.nodeName())

        if name not in self.dialog.ui.widget.subItems:
            return

        index = self.view.currentIndex()
        if not index.isValid():
            return
        sel = index.internalPointer()
        if not sel:
            return

        node = sel.node

        self.dialog.ui.widget.node = node
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.dialog.ui.widget.appendElement(clipNode, index)

        self.view.model().dataChanged.emit(index, index)

        self.view.expand(index)
        self.fetchElements()
        return True

    # creates the component item with the given name in the component tree
    # \param name component item name
    # \returns component DOM node related to the new item
    def addItem(self, name):
        if name not in self._tagClasses.keys():
            return
        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget:
            return
        if not hasattr(self.dialog.ui.widget, 'subItems') \
                or name not in self.dialog.ui.widget.subItems:
            return
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel or not index.isValid():
            return
        node = sel.node
        self.dialog.ui.widget.node = node
        child = self.dialog.ui.widget.root.createElement(str(name))
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        status = self.dialog.ui.widget.appendElement(child, index)
        self.view.model().dataChanged.emit(index, index)
        self.view.expand(index)
        self.fetchElements()
        if status:
            return child

    # removes the currenct component tree item if possible
    # \returns True on success
    def removeSelectedItem(self):

        if not self.view or not self.view.model():
            return
        dialog = True if self.dialog and self.dialog.ui \
            and self.dialog.ui.widget else False
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel or not index.isValid():
            return

        node = sel.node

        clipboard = QApplication.clipboard()
        clipboard.setText(self._nodeToString(node))

        if hasattr(self.dialog.ui.widget, "node") and dialog:
            self.dialog.ui.widget.node = node.parentNode()

        if self.view is not None and self.view.model() is not None:
            DomTools.removeNode(node, index.parent(), self.view.model())

        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.view.model().dataChanged.emit(index, index)
        if index.parent().isValid():
            self.view.model().dataChanged.emit(
                index.parent(), index.parent())

            index = self.view.currentIndex()
            self.tagClicked(index)
        else:
            self.tagClicked(QModelIndex())

        self.fetchElements()
        return True

    # copies the currenct component tree item if possible
    # \returns True on success
    def copySelectedItem(self):
        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget:
            return
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel:
            return

        node = sel.node

        clipboard = QApplication.clipboard()
        clipboard.setText(self._nodeToString(node))
        self.fetchElements()
        return True

    # creates GUI
    # It calls setupUi and creates required action connections
    def createGUI(self):
        self.dialog = ComponentDlg(self, self.parent)
        self.dialog.ui.setupUi(self.dialog)
        self.view = self.dialog.ui.view

        self.updateForm()
        self.connectView()

    # connects the view and model into resize and click command
    def connectView(self):
        try:
            self.view.selectionModel().currentChanged.disconnect(
                self.tagClicked)
        except Exception:
            pass
        self.view.selectionModel().currentChanged.connect(
            self.tagClicked)
        # self.dialog.disconnect(
        #     self.view.selectionModel(),
        #     SIGNAL("currentChanged(QModelIndex,QModelIndex)"),
        #     self.tagClicked)
        # self.parent.connect(
        #     self.view.selectionModel(),
        #     SIGNAL("currentChanged(QModelIndex,QModelIndex)"),
        #     self.tagClicked)
        try:
            self.view.expanded.disconnect(self._resizeColumns)
        except Exception:
            pass
        self.view.expanded.connect(self._resizeColumns)
        # self.dialog.disconnect(
        #     self.view, SIGNAL("expanded(QModelIndex)"),
        #     self._resizeColumns)
        # self.parent.connect(
        #     self.view, SIGNAL("expanded(QModelIndex)"),
        #     self._resizeColumns)
        try:
            self.view.collapsed.disconnect(self._resizeColumns)
        except Exception:
            pass
        self.view.collapsed.connect(self._resizeColumns)
        # self.dialog.disconnect(
        #     self.view, SIGNAL("collapsed(QModelIndex)"),
        #     self._resizeColumns)
        # self.parent.connect(
        #     self.view, SIGNAL("collapsed(QModelIndex)"),
        #     self._resizeColumns)

    # connects the save action and stores the apply action
    # \param externalApply apply action
    # \param externalSave save action
    # \param externalClose close action
    # \param externalStore store action
    # \param externalDSLink dsource link action
    def connectExternalActions(self, externalApply=None, externalSave=None,
                               externalClose=None, externalStore=None,
                               externalDSLink=None):
        if externalSave and self.externalSave is None:
            self.dialog.ui.savePushButton.clicked.connect(
                externalSave)
            # self.parent.connect(
            #     self.dialog.ui.savePushButton, SIGNAL("clicked()"),
            #     externalSave)
            self.externalSave = externalSave
        if externalStore and self.externalStore is None:
            self.dialog.ui.storePushButton.clicked.connect(
                externalStore)
            # self.parent.connect(
            #     self.dialog.ui.storePushButton, SIGNAL("clicked()"),
            #     externalStore)
            self.externalStore = externalStore
        if externalClose and self.externalClose is None:
            self.dialog.ui.closePushButton.clicked.connect(
                externalClose)
            # self.parent.connect(
            #     self.dialog.ui.closePushButton, SIGNAL("clicked()"),
            #     externalClose)
            self.externalClose = externalClose
        if externalApply and self.externalApply is None:
            self.externalApply = externalApply
        if externalDSLink and self.externalDSLink is None:
            self.externalDSLink = externalDSLink

    # reconnects save actions
    # \brief It reconnects the save action
    def reconnectSaveAction(self):
        self.connectView()
        if self.externalSave:
            try:
                self.dialog.ui.savePushButton.clicked.disconnect(
                    self.externalSave)
            except Exception:
                pass
            self.dialog.ui.savePushButton.clicked.connect(
                self.externalSave)
            # self.dialog.disconnect(
            #     self.dialog.ui.savePushButton, SIGNAL("clicked()"),
            #     self.externalSave)
            # self.parent.connect(
            #     self.dialog.ui.savePushButton, SIGNAL("clicked()"),
            #     self.externalSave)
        if self.externalStore:
            try:
                self.dialog.ui.storePushButton.clicked.disconnect(
                    self.externalStore)
            except Exception:
                pass
            self.dialog.ui.storePushButton.clicked.connect(
                self.externalStore)
            # self.dialog.disconnect(
            #     self.dialog.ui.storePushButton, SIGNAL("clicked()"),
            #     self.externalStore)
            # self.parent.connect(
            #     self.dialog.ui.storePushButton, SIGNAL("clicked()"),
            #     self.externalStore)
        if self.externalClose:
            try:
                self.dialog.ui.closePushButton.clicked.disconnect(
                    self.externalClose)
            except Exception:
                pass
            self.dialog.ui.closePushButton.clicked.connect(
                self.externalClose)
            # self.dialog.disconnect(
            #     self.dialog.ui.closePushButton, SIGNAL("clicked()"),
            #     self.externalClose)
            # self.parent.connect(
            #     self.dialog.ui.closePushButton, SIGNAL("clicked()"),
            #     self.externalClose)

    # switches between all attributes in the try or only type attribute
    # \param status all attributes are shown if True
    def viewAttributes(self, status):
        if status == self._allAttributes:
            return
        self._allAttributes = status
        if hasattr(self, "view") and self.dialog:
            cNode = self._getCurrentNode()
            model = self.view.model()
            model.setAttributeView(self._allAttributes)
#             self.view.reset()
            newModel = ComponentModel(
                self.document, self._allAttributes, self.parent)
            self.view.setModel(newModel)
            self.connectView()
            self._hideFrame()
            if cNode:
                self._showNodes([cNode])

    # sets selected component item in the item frame
    # \brief It is executed  when component tree item is selected
    # \param index of component tree item
    def tagClicked(self, index):
        if not index.isValid() or not self.dialog:
            logger.warn("Not valid index")
            return
        self._currentTag = index
        item = self._currentTag.internalPointer()
        if item is None:
            logger.warn("Not valid index item")
            return

        node = item.node
        nNode = node.nodeName()

        if not self.dialog.ui:
            logger.warn("Dialog does not exist")
            return

        if self.dialog.ui.widget:
            self.dialog.ui.widget.setVisible(False)
        if unicode(nNode) in self._tagClasses.keys():
            if self.dialog.ui and self.dialog.ui.widget:
                if hasattr(self.dialog.ui.widget, "widget"):
                    self.dialog.ui.widget.widget.hide()
                else:
                    self.dialog.ui.widget.hide()

            self.dialog.ui.frame.hide()
            self._frameLayout.removeWidget(self.dialog.ui.widget)
            self.dialog.ui.widget = self._tagClasses[
                unicode(nNode)](self.dialog)
            self.dialog.ui.widget.root = self.document
            self.dialog.ui.widget.setFromNode(node)
            self.dialog.ui.widget.createGUI()
            if hasattr(self.dialog.ui.widget, "connectExternalActions"):
                self.dialog.ui.widget.connectExternalActions(
                    externalApply=self.externalApply,
                    externalDSLink=self.externalDSLink)
            if hasattr(self.dialog.ui.widget, "treeMode"):
                self.dialog.ui.widget.treeMode()
            self.dialog.ui.widget.view = self.view
            self.dialog.ui.view = self.view
            self._frameLayout.addWidget(self.dialog.ui.widget)
            self.dialog.ui.widget.show()
            self.dialog.ui.frame.show()
        else:
            if self.dialog.ui.widget:
                self.dialog.ui.widget.hide()
            self.dialog.ui.widget = None

    # opens context Menu
    # \param position in the component tree
    def _openMenu(self, position):
        if not self.dialog:
            return
        index = self.view.indexAt(position)
        if index.isValid():
            self.tagClicked(index)

        menu = QMenu()
        for action in self._actions:
            if action is None:
                menu.addSeparator()
            else:
                menu.addAction(action)
        menu.exec_(self.view.viewport().mapToGlobal(position))

    # sets up context menu
    # \param actions list of the context menu actions
    def addContextMenu(self, actions):
        self.view.setContextMenuPolicy(Qt.CustomContextMenu)
        self.view.customContextMenuRequested.connect(self._openMenu)
        self._actions = actions

    # resizes column size after tree item expansion
    # \param index index of the expanded item
    def _resizeColumns(self, index):
        for column in range(self.view.model().columnCount(index)):
            self.view.resizeColumnToContents(column)

    # sets name and directory of the current component
    # \param name component name
    # \param directory component directory
    def setName(self, name, directory=None):
        fi = None
        dr = ""
        if self._componentFile:
            fi = QFileInfo(self._componentFile)
        if directory is None and fi:
            dr = unicode(fi.dir().path())
        elif directory:
            dr = unicode(directory)
        elif self.directory:
            dr = self.directory
        else:
            dr = os.getcwd()

        self._componentFile = os.path.join(dr, name + ".xml")
        self.name = name
        self._xmlPath = self._componentFile

    # loads the component from the file
    # \param filePath xml file name with full path
    def load(self, filePath=None):

        if not filePath:
            if not self.name:
                self._componentFile = unicode(QFileDialog.getOpenFileName(
                    self.parent, "Open File", self._xmlPath,
                    "XML files (*.xml);;HTML files (*.html);;"
                    "SVG files (*.svg);;User Interface files (*.ui)")[0])
            else:
                self._componentFile = os.path.join(
                    self.directory, self.name + ".xml")
        else:
            self._componentFile = filePath
        if self._componentFile:
            try:
                fh = QFile(self._componentFile)
                if fh.open(QIODevice.ReadOnly):
                    self._loadFromString(fh)
                    self._xmlPath = self._componentFile
                    fi = QFileInfo(self._componentFile)
                    self.name = unicode(fi.fileName())

                    if self.name[-4:] == '.xml':
                        self.name = self.name[:-4]
                    self.savedXML = self.get()
                    return self._componentFile
                else:
                    QMessageBox.warning(
                        self.parent, "Cannot open the file",
                        "Cannot open the file: %s" % (self._componentFile))

            except (IOError, OSError, ValueError) as e:
                error = "Failed to load: %s" % e
                QMessageBox.warning(self.parent, "Loading problem",
                                    error)

                logger.warn(error)
            finally:
                self.fetchElements()
                if fh is not None:
                    fh.close()

    # sets component from XML string and reset component file name
    # \param xml XML string
    # \param new logical variableset to True if element is not saved
    def set(self, xml, new=False):
        self._componentFile = os.path.join(
            self.directory, self.name + ".xml")
        self._loadFromString(xml)
        self._xmlPath = self._componentFile
        self.savedXML = self.get()
        self.fetchElements()
        return self._componentFile

    # sets component from XML string
    # \param xml XML string
    def _loadFromString(self, xml):
        self.document = QDomDocument()

        if not self.document.setContent(xml):
            raise ValueError("could not parse XML")
        children = self.document.childNodes()

        j = 0
        for _ in range(children.count()):
            ch = children.item(j)
            if str(ch.nodeName()).strip() in \
                    ['xml', 'xml-stylesheet', "#comment"]:
                self.document.removeChild(ch)
            else:
                j += 1
        if self.dialog and self.dialog.ui:
            newModel = ComponentModel(
                self.document, self._allAttributes, self.parent)
            self.view.setModel(newModel)
            self.connectView()

    # loads the component item from the xml file
    # \param filePath xml file name with full path
    def loadComponentItem(self, filePath=None):

        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget \
                or not hasattr(self.dialog.ui.widget, "subItems") \
                or "component" not in self.dialog.ui.widget.subItems:
            return
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel or not index.isValid():
            return
        node = sel.node

        if not filePath:
            itemFile = unicode(
                QFileDialog.getOpenFileName(
                    self.parent, "Open File", self._xmlPath,
                    "XML files (*.xml);;HTML files (*.html);;"
                    "SVG files (*.svg);;User Interface files (*.ui)")[0])
        else:
            itemFile = filePath

        if itemFile:
            try:
                fh = QFile(itemFile)
                if fh.open(QIODevice.ReadOnly):
                    root = QDomDocument()
                    if not root.setContent(fh):
                        raise ValueError("could not parse XML")
                    definition = root.firstChildElement(str("definition"))
                    if definition.nodeName() != "definition":
                        QMessageBox.warning(
                            self.parent, "Corrupted SubComponent",
                            "Component %s without <definition> tag" % itemFile)
                        return
                    child = definition.firstChild()
                    self.dialog.ui.widget.node = node

                    if index.column() != 0:
                        index = self.view.model().index(
                            index.row(), 0, index.parent())
                    while not child.isNull():
                        child2 = self.document.importNode(child, True)
                        self.dialog.ui.widget.appendElement(child2, index)

                        child = child.nextSibling()

                self.view.model().dataChanged.emit(
                    index, index)
                self.view.expand(index)

            except (IOError, OSError, ValueError) as e:
                error = "Failed to load: %s" % e
                QMessageBox.warning(self.parent, "Loading problem",
                                    error)
                logger.warn(error)
            finally:
                if fh is not None:
                    fh.close()

        self.fetchElements()
        return True

    # loads the datasource item from the xml file
    # \param filePath xml file name with full path
    def loadDataSourceItem(self, filePath=None):

        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget \
                or not hasattr(self.dialog.ui.widget, "subItems") \
                or "datasource" not in self.dialog.ui.widget.subItems:
            return
        child = self.dialog.ui.widget.node.firstChild()
        while not child.isNull():
            if child.nodeName() == 'datasource':
                QMessageBox.warning(
                    self.parent, "DataSource exists",
                    "To add a new datasource please remove the old one")
                return
            child = child.nextSibling()

        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel or not index.isValid():
            return
        node = sel.node

        if not filePath:
            dsFile = unicode(
                QFileDialog.getOpenFileName(
                    self.parent, "Open File", self._dsPath,
                    "XML files (*.xml);;HTML files (*.html);;"
                    "SVG files (*.svg);;User Interface files (*.ui)")[0])
        else:
            dsFile = filePath
        if dsFile:
            try:
                fh = QFile(dsFile)
                if fh.open(QIODevice.ReadOnly):
                    root = QDomDocument()
                    if not root.setContent(fh):
                        raise ValueError("could not parse XML")
                    ds = DomTools.getFirstElement(root, "datasource")

                    if ds:
                        if index.column() != 0:
                            index = self.view.model().index(
                                index.row(), 0, index.parent())
                        self.dialog.ui.widget.node = node
                        ds2 = self.document.importNode(ds, True)
                        self.dialog.ui.widget.appendElement(ds2, index)
                    else:
                        QMessageBox.warning(
                            self.parent, "Corrupted DataSource ",
                            "Missing <datasource> tag in %s" % dsFile)

                self.view.model().dataChanged.emit(
                    index, index)
                self.view.expand(index)
                self._dsPath = dsFile

            except (IOError, OSError, ValueError) as e:
                error = "Failed to load: %s" % e
                logger.warn(error)
            finally:
                if fh is not None:
                    fh.close()

        self.fetchElements()
        return True

    # add the datasource into the component tree
    # \param dsNode datasource DOM node
    def addDataSourceItem(self, dsNode):

        if dsNode.nodeName() != 'datasource':
            return

        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget \
                or not hasattr(self.dialog.ui.widget, "subItems") \
                or "datasource" not in self.dialog.ui.widget.subItems:
            return

        child = self.dialog.ui.widget.node.firstChild()
        while not child.isNull():
            if child.nodeName() == 'datasource':
                QMessageBox.warning(
                    self.parent, "DataSource exists",
                    "To add a new datasource please remove the old one")
                return
            child = child.nextSibling()

        index = self.view.currentIndex()
        if not index.isValid():
            return

        sel = index.internalPointer()
        if not sel:
            return

        node = sel.node

        self.dialog.ui.widget.node = node
        dsNode2 = self.document.importNode(dsNode, True)
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.dialog.ui.widget.appendElement(dsNode2, index)

        self.view.model().dataChanged.emit(index, index)
        self.view.expand(index)
        self.fetchElements()
        return True

    # link the datasource into the component tree
    # \param dsName datasource name
    def linkDataSourceItem(self, dsName):

        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget \
                or not hasattr(self.dialog.ui.widget, "subItems") \
                or "datasource" not in self.dialog.ui.widget.subItems:
            return

        child = self.dialog.ui.widget.node.firstChild()
        while not child.isNull():
            if child.nodeName() == 'datasource':
                QMessageBox.warning(
                    self.parent, "DataSource exists",
                    "To link a new datasource please remove the old one")
                return
            child = child.nextSibling()

        index = self.view.currentIndex()
        if not index.isValid():
            return

        sel = index.internalPointer()
        if not sel:
            return

        if hasattr(self.dialog.ui.widget, "linkDataSource"):
            self.dialog.ui.widget.linkDataSource(dsName)

        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())

        self.view.model().dataChanged.emit(index, index)
        self.view.expand(index)
        self.fetchElements()
        return True

    # accepts merger dialog and interrupts merging
    # \brief It is connected to closing Merger dialog
    def _closeMergerDlg(self):
        if self._mergerdlg:
            self._mergerdlg.accept()

            self._interruptMerger()

    # interrupts merging
    # \brief It sets running flag of Merger to False
    def _interruptMerger(self):
        if self._merger:
            self._merger.running = False

    # merges the component tree
    # \returns True on success
    def merge(self, showMergerDlg=True):
        document = None
        dialog = False

        if showMergerDlg:
            self._mergerdlg = MergerDlg(self.parent)
            self._mergerdlg.createGUI()
            self._mergerdlg.finished.connect(
                self._interruptMerger)
            self._mergerdlg.interruptButton.clicked.connect(
                self._interruptMerger)
            # self.parent.connect(
            #     self._mergerdlg, SIGNAL("finished(int)"),
            #     self._interruptMerger)
            # self.parent.connect(
            #     self._mergerdlg.interruptButton, SIGNAL("clicked()"),
            #     self._interruptMerger)

        try:
            if self.view and self.dialog and self.dialog.ui \
                    and self.view.model():
                dialog = True
        except Exception:
            pass

        if not self.document:
            self._merged = False
            return
        try:
            self._merger = Merger(self.document)
            if showMergerDlg:
                self._merger.finished.connect(self._closeMergerDlg)
                # self.parent.connect(
                #     self._merger, SIGNAL("finished"), self._closeMergerDlg)

            cNode = self._getCurrentNode()
            if cNode:
                self._merger.selectedNode = cNode

            document = self.document
            self.document = QDomDocument()
            if dialog:
                newModel = ComponentModel(
                    self.document, self._allAttributes, self.parent)
                self.view.setModel(newModel)
                self.view.reset()
                self._hideFrame()
                self.connectView()

            self._merger.start()

            if showMergerDlg:
                self._mergerdlg.exec_()

            if self._merger:
                self._merger.wait()

            if showMergerDlg:
                self._closeMergerDlg()

            if self._merger and self._merger.exception is not None:
                raise self._merger.exception

            self._merged = True
            if dialog:
                newModel = ComponentModel(
                    document, self._allAttributes, self.parent)
            self.document = document
            if dialog:
                self.view.setModel(newModel)
                self._hideFrame()
                self.connectView()

                self.connectView()
                if hasattr(self._merger, "selectedNode") \
                        and self._merger.selectedNode:
                    self._showNodes([self._merger.selectedNode])

            self._merger = None

        except IncompatibleNodeError as e:
            logger.warn("Error in Merging: %s" % unicode(e.value))
            self._merger = None
            self._merged = False
            if dialog:
                newModel = ComponentModel(
                    document, self._allAttributes, self.parent)
            self.document = document
            if dialog:
                self.view.setModel(newModel)
                self._hideFrame()
                self.connectView()
                if hasattr(e, "nodes") and e.nodes:
                    self._showNodes(e.nodes)
            if dialog:
                QMessageBox.warning(
                    self.parent, "Merging problem",
                    "Error in %s Merging: %s" % (
                        unicode(self.name), unicode(e.value)))
        except Exception as e:
            logger.warn("Exception: %s" % unicode(e))
            self._merged = False
            if dialog:
                newModel = ComponentModel(
                    document, self._allAttributes, self.parent)
                self.document = document
                self.view.setModel(newModel)
                self._hideFrame()
                self.connectView()
            if dialog:
                QMessageBox.warning(self.parent, "Warning",
                                    "%s" % unicode(e))
        return self._merged

    # hides the component item frame
    # \brief It puts an empty widget into the widget frame
    def _hideFrame(self):
        if self.dialog and self.dialog.ui:
            if self.dialog.ui.widget:
                if hasattr(self.dialog.ui.widget, "widget"):
                    self.dialog.ui.widget.widget.setVisible(False)
                else:
                    self.dialog.ui.widget.setVisible(False)
            self.dialog.ui.widget = QWidget(self.dialog)
            self._frameLayout.addWidget(self.dialog.ui.widget)
            self.dialog.ui.widget.show()
            self.dialog.ui.frame.show()

    # creates the new empty header
    # \brief It clean the DOM tree and put into it xml and definition nodes
    def createHeader(self):
        self.document = QDomDocument()
        self.document = self.document

        definition = self.document.createElement(str("definition"))
        self.document.appendChild(definition)
        if self.dialog and self.dialog.ui:
            newModel = ComponentModel(
                self.document, self._allAttributes, self.parent)
            self.view.setModel(newModel)
            self.connectView()
        self.fetchElements()
        self._hideFrame()

    # fetches tags with a given name  from the node branch
    # \param name of the
    # \returns dictionary with the required tags
    def _getTags(self, node, tagName):
        ds = {}
        if node:
            children = node.childNodes()
            for c1 in range(children.count()):
                child = children.item(c1)
                if child.nodeName() == tagName:
                    attr = child.attributes()
                    if attr.contains("name"):
                        name = unicode(attr.namedItem("name").nodeValue())
                    else:
                        self._tagCnt += 1
                        name = "__datasource__%s" % self._tagCnt
                    ds[name] = self._nodeToString(child)

            child = node.firstChild()
            if child:
                while not child.isNull():
                    childElem = child.toElement()
                    if childElem:
                        ds.update(self._getTags(childElem, tagName))
                    child = child.nextSibling()

        return ds

    # fetches the datasources from the component
    # \returns dictionary with datasources
    def getDataSources(self):
        ds = {}
        if hasattr(self.document, "toString"):
            self._tagCnt = 0
            ds.update(self._getTags(self.document, "datasource"))
        return ds

    # fetches the datasources from the component
    # \returns dictionary with datasources
    def getCurrentDataSource(self):
        ds = {}

        if not self.view or not self.dialog or not self.view.model() \
                or not self.dialog.ui or not self.dialog.ui.widget:
            return ds
        index = self.view.currentIndex()
        sel = index.internalPointer()
        if not sel:
            return ds
        if sel.node.nodeName() != "datasource":
            return ds

        node = sel.node

        attr = node.attributes()
        if attr.contains("name"):
            name = unicode(attr.namedItem("name").nodeValue())
        else:
            name = "__datasource__"
        ds[name] = self._nodeToString(node)
        return ds

    # provides the component in xml string
    # \param indent number of added spaces during pretty printing
    # \returns xml string
    def get(self, indent=0):
        if hasattr(self.document, "toString"):
            processing = self.document.createProcessingInstruction(
                "xml", "version='1.0'")
            self.document.insertBefore(processing, self.document.firstChild())
            string = unicode(self.document.toString(indent))
            self.document.removeChild(processing)
            return string

    # saves the component
    # \brief It saves the component in the xml file
    def save(self):
        if not self._merged:
            QMessageBox.warning(self.parent, "Saving problem",
                                "Document not merged")
            return
        error = None
        if self._componentFile is None:
            self.setName(self.name, self.directory)
        fpath = os.path.join(self.directory, self.name + ".xml")
        logger.info("saving %s" % fpath)
        try:
            fh = QFile(fpath)
            if not fh.open(QIODevice.WriteOnly):
                raise IOError(unicode(fh.errorString()))
            stream = QTextStream(fh)
            string = self.get(2)
            if string:
                stream << string
            else:
                raise ValueError("Empty component")

            self.savedXML = self.get()
        except (IOError, OSError, ValueError) as e:
            error = "Failed to save: %s Please try to use Save as "\
                "or change the component directory" % e
            QMessageBox.warning(self.parent, "Saving problem",
                                error)
            logger.warn(error)
        finally:
            if fh is not None:
                fh.close()
        if not error:
            return True

    # asks if component should be removed from the component list
    # \brief It is called on removing  the component from the list
    def _close(self):
        if QMessageBox.question(
                self.parent, "Close component",
                "Would you like to close the component ?",
                QMessageBox.Yes | QMessageBox.No) == QMessageBox.No:
            return
        self.dialog.reject()

    # provides the component name with its path
    # \returns component name with its path
    def getNewName(self):
        if self._componentFile is None:
            self.setName(self.name, self.directory)

        self._componentFile = unicode(
            QFileDialog.getSaveFileName(
                self.parent, "Save Component As ...", self._componentFile,
                "XML files (*.xml);;HTML files (*.html);;"
                "SVG files (*.svg);;User Interface files (*.ui)")[0])
        return self._componentFile


# test function
def test():
    import sys

    # Qt application
    app = QApplication(sys.argv)
    component = Component()
    component.createGUI()
    component.dialog.resize(680, 560)
    component.createHeader()
    component.dialog.show()
    component.dialog.setWindowTitle("%s [Component]" % component.name)

    app.exec_()


if __name__ == "__main__":
    test()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file CreatorDlg.py
# Component Creator dialog class

""" server creator widget """
import os
import sys

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QTableWidgetItem, QMessageBox
from PyQt5 import uic

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "creatordlg.ui"))

_stdformclass, _stdbaseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "stdcreatordlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a component creator dialog
class CreatorDlg(QDialog):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(CreatorDlg, self).__init__(parent)

        # host name of the configuration server
        self.components = []
        self.componentName = None
        # user interface
        self.ui = _formclass()
        self.action = ''

    # creates GUI
    # \brief It updates GUI and creates creatorion for required actions
    def createGUI(self):
        self.ui.setupUi(self)
        self.updateForm()

        self.ui.savePushButton.clicked.connect(self.savePressed)
        self.ui.storePushButton.clicked.connect(self.storePressed)
        self.ui.applyPushButton.clicked.connect(self.applyPressed)
        self.ui.cancelPushButton.clicked.connect(self.reject)

    # updates the connect dialog
    # \brief It sets initial values of the connection form
    def updateForm(self):
        self.ui.compComboBox.clear()
        if self.components:
            self.ui.compComboBox.addItems(self.components)

    def savePressed(self):
        self.componentName = unicode(self.ui.compComboBox.currentText())
        self.action = 'SAVE'
        QDialog.accept(self)

    def storePressed(self):
        self.componentName = unicode(self.ui.compComboBox.currentText())
        self.action = 'STORE'
        QDialog.accept(self)

    def applyPressed(self):
        self.componentName = unicode(self.ui.compComboBox.currentText())
        self.action = 'APPLY'
        QDialog.accept(self)


# dialog defining a component creator dialog
class StdCreatorDlg(QDialog):

    # constructor
    # \param parent patent instance
    def __init__(self, creator, parent=None):
        super(StdCreatorDlg, self).__init__(parent)

        # host name of the configuration server
        self.__parent = parent
        self.__types = []
        self.__creator = creator
        self.componentType = None
        self.componentName = None
        # user interface
        self.ui = _stdformclass()
        self.action = ''
        self.__vars = {}
        self.__pardesc = {}
        self.__oldvars = {}

    # creates GUI
    # \brief It updates GUI and creates creatorion for required actions
    def createGUI(self):
        self.ui.setupUi(self)
        self.__types = list(self.__creator.listcomponenttypes() or [])
        self.updateForm()
        self.__updateUi()

        self.ui.savePushButton.clicked.connect(
            self.savePressed)
        self.ui.linkPushButton.clicked.connect(
            self.linkPressed)
        self.ui.storePushButton.clicked.connect(
            self.storePressed)
        self.ui.applyPushButton.clicked.connect(
            self.applyPressed)
        self.ui.cancelPushButton.clicked.connect(
            self.reject)
        self.ui.varTableWidget.itemChanged.connect(
            self.__tableItemChanged)
        self.ui.cpNameLineEdit.textEdited[str].connect(
            self.__updateUi)
        self.ui.cpTypeComboBox.currentIndexChanged[str].connect(
            self.__currentIndexChanged)

    # updates the connect dialog
    # \brief It sets initial values of the connection form
    def updateForm(self):
        self.ui.cpTypeComboBox.clear()
        if self.__types:
            self.ui.cpTypeComboBox.addItems(self.__types)
        self.updateParams()

    def updateParams(self):
        self.componentType = unicode(self.ui.cpTypeComboBox.currentText())
        self.__creator.options.cptype = self.componentType or None
        if self.componentType:
            self.__pardesc = self.__creator.listcomponentvariables()
            self.__vars = {}
            for var in sorted(self.__pardesc.keys()):
                desc = self.__pardesc[var]
                if not var.startswith('__') and not var.endswith('__'):
                    if var not in self.__vars or self.__vars[var] is None:
                        if var in self.__oldvars and \
                           self.__oldvars[var] is not None:
                            self.__vars[var] = self.__oldvars[var]
                        else:
                            self.__vars[var] = desc['default']
                            self.__oldvars[var] = self.__vars[var]

        self.populateVars()

    def populateArgs(self):
        args = []
        for name, value in self.__vars.items():
            if value:
                args.extend([name, value])
        self.__creator.args = args

    def savePressed(self):
        self.componentName = unicode(self.ui.cpNameLineEdit.text())
        self.__creator.options.component = self.componentName or None
        self.populateArgs()
        self.action = 'SAVE'
        QDialog.accept(self)

    def storePressed(self):
        self.componentName = unicode(self.ui.cpNameLineEdit.text())
        self.__creator.options.component = self.componentName or None
        self.populateArgs()
        self.action = 'STORE'
        QDialog.accept(self)

    def applyPressed(self):
        self.componentName = unicode(self.ui.cpNameLineEdit.text())
        self.__creator.options.component = self.componentName or None
        self.populateArgs()
        self.action = 'APPLY'
        QDialog.accept(self)

    def linkPressed(self):
        dsname = self.__parent.currentDataSourceName()
        if not dsname:
            QMessageBox.warning(
                self, "DataSource not selected",
                "Please select the required datasource from the list")
            return

        var = self.__currentTableVar()
        if unicode(var) not in self.__vars.keys():
            QMessageBox.warning(
                self, "Variable not selected",
                "Please select the required variable from the table")
            return
            return
        self.__vars[unicode(var)] = unicode(dsname)
        self.__oldvars[unicode(var)] = self.__vars[unicode(var)]
        self.populateVars()

    # calls updateUi when the name text is changing
    # \param text the edited text
    def __currentIndexChanged(self, text):
        self.updateParams()

    # takes a name of the current variable
    # \returns name of the current variable
    def __currentTableVar(self):
        item = self.ui.varTableWidget.item(
            self.ui.varTableWidget.currentRow(), 0)
        if item is None:
            return None
        return item.data(Qt.UserRole)

    # changes the current value of the variable
    # \brief It changes the current value of the variable
    #        and informs the user that variable names arenot editable
    def __tableItemChanged(self, item):
        var = self.__currentTableVar()
        if unicode(var) not in self.__vars.keys():
            return
        column = self.ui.varTableWidget.currentColumn()
        if column == 1:
            self.__vars[unicode(var)] = unicode(item.text())
            self.__oldvars[unicode(var)] = self.__vars[unicode(var)]
        if column == 0:
            QMessageBox.warning(
                self, "Variable name is not editable",
                "You cannot change it")
        if column == 3:
            QMessageBox.warning(
                self, "Variable descritpion is not editable",
                "You cannot change it")
        self.populateVars()

    # fills in the variable table
    # \param selectedVariable selected variable
    def populateVars(self, selectedVar=None):
        selected = None
        self.ui.varTableWidget.clear()
        self.ui.varTableWidget.setSortingEnabled(False)
        self.ui.varTableWidget.setRowCount(len(self.__vars))
        headers = ["Name", "Value", "Info"]
        self.ui.varTableWidget.setColumnCount(len(headers))
        self.ui.varTableWidget.setHorizontalHeaderLabels(headers)
        for row, name in enumerate(sorted(self.__vars.keys())):
            item = QTableWidgetItem(name)
            item.setData(Qt.UserRole, (name))
            flags = item.flags()
            flags ^= Qt.ItemIsEditable
            item.setFlags(flags)
            self.ui.varTableWidget.setItem(row, 0, item)
            item2 = QTableWidgetItem(self.__vars[name] or "")
            self.ui.varTableWidget.setItem(row, 1, item2)
            item3 = QTableWidgetItem(self.__pardesc[name]['doc'] or "")
            flags = item3.flags()
            flags &= ~Qt.ItemIsEnabled
            item3.setFlags(flags)
            self.ui.varTableWidget.setItem(row, 2, item3)
            if selectedVar is not None and selectedVar == name:
                selected = item2
        self.ui.varTableWidget.setSortingEnabled(True)
        self.ui.varTableWidget.resizeColumnsToContents()
        self.ui.varTableWidget.horizontalHeader()\
            .setStretchLastSection(True)
        if selected is not None:
            selected.setSelected(True)
            self.ui.varTableWidget.setCurrentItem(selected)

    # updates group user interface
    # \brief It sets enable or disable the OK button
    def __updateUi(self):
        enable = bool(self.ui.cpNameLineEdit.text())
        self.ui.applyPushButton.setEnabled(enable)
        self.ui.storePushButton.setEnabled(enable)
        self.ui.savePushButton.setEnabled(enable)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # connect form
    form = CreatorDlg()
    form.createGUI()
    form.show()
    app.exec_()

    if form.result():
        if form.device:
            logger.info("Connect: %s , %s , %s" %
                        (form.device, form.host, form.port))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file FileSlots.py
# user pool commands of GUI application

""" File slots """

from PyQt5.QtGui import QKeySequence

from .FileCommands import (
    ComponentOpen,
    DataSourceOpen,
    ComponentSave,
    ComponentSaveAll,
    ComponentSaveAs,
    ComponentChangeDirectory,
    DataSourceSaveAll,
    DataSourceSave,
    DataSourceSaveAs,
    ComponentReloadList,
    DataSourceReloadList,
    DataSourceChangeDirectory
)

from .EditCommands import (
    ComponentEdit,
    DataSourceEdit,
    DataSourceApply
)

from .ItemCommands import (
    ComponentApplyItem,
    ComponentMerge
)

from .ListCommands import (
    ComponentListChanged,
    DataSourceListChanged
)


# stack with the application commands
class FileSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # action data
        self.actions = {
            "actionLoad": [
                "&Load...", "componentOpen",
                QKeySequence.Open, "componentopen",
                "Load an existing component"],
            "actionLoadDataSource": [
                "&Load DataSource...", "dsourceOpen",
                "Ctrl+Shift+O", "dsourceopen",
                "Load an existing data source"],
            "actionSave": [
                "&Save", "componentSave",
                QKeySequence.Save, "componentsave",
                "Write the component into a file"],
            "actionSaveDataSource": [
                "&Save DataSource", "dsourceSave",
                "Ctrl+Shift+S", "dsourcesave",
                "Write the data source into a file"],
            "actionSaveAs": [
                "Save &As...", "componentSaveAs",
                "", "componentsaveas",
                "Write the component into a file as ..."],
            "actionSaveDataSourceAs": [
                "Save DataSource &As...", "dsourceSaveAs",
                "", "dsourcesaveas",
                "Write the data source  in a file as ..."],
            "actionSaveAll": [
                "Save All", "componentSaveAll",
                "", "componentsaveall", "Write all components into files"],
            "actionSaveAllDataSources": [
                "Save All DataSources", "dsourceSaveAll",
                "", "dsourcessaveall", "Write all data sources in files"],
            "actionReloadDataSourceList": [
                "Reload DataSource List", "dsourceReloadList",
                "", "dsourcereloadlist", "Reload the data-source list"],
            "actionReloadList": [
                "Reload List", "componentReloadList",
                "", "componentreloadlist", "Reload the component list"],
            "actionChangeDirectory": [
                "Change Directory...", "componentChangeDirectory",
                "", "componentrechangedirectory",
                "Change the component list directory"],
            "actionChangeDataSourceDirectory": [
                "Change DataSource Directory...", "dsourceChangeDirectory",
                "", "dsourcerechangedirectory",
                "Change the data-source list directory"]
        }

    # open component action
    # \brief It opens component from the file
    def componentOpen(self):
        cmd = ComponentOpen(self.main)
        self.undoStack.push(cmd)

    # open datasource action
    # \brief It opens datasource from the file
    def dsourceOpen(self):
        cmd = DataSourceOpen(self.main)
        self.undoStack.push(cmd)

    # save component action
    # \brief It saves the current component
    def componentSave(self, focus=True):
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentMerge(self.main)
        self.undoStack.push(cmd)
        cmd = ComponentSave(self.main)
        cmd.redo()
        if focus:
            self.main.componentList.setItemFocus()

    # save component action executed by button
    # \brief It saves the current component executed by button
    def componentSaveButton(self):
        if self.main.updateComponentListItem():
            cmd = ComponentApplyItem(self.main)
            self.undoStack.push(cmd)
            self.componentSave(False)

    # save datasource item action
    # \brief It saves the changes in the current datasource item
    def dsourceSave(self, focus=True):
        cmd = DataSourceEdit(self.main)
        cmd.redo()
        cmd = DataSourceApply(self.main)
        self.undoStack.push(cmd)
        cmd = DataSourceSave(self.main)
        cmd.redo()
        if focus:
            self.main.sourceList.setItemFocus()

    # save datasource item action executed by button
    # \brief It saves the changes in the current datasource item executed
    #        by button
    def dsourceSaveButton(self):
        if self.main.updateDataSourceListItem():
            self.dsourceSave(False)

    # save component item as action
    # \brief It saves the changes in the current component item with a new name
    def componentSaveAs(self):
        cmd = ComponentApplyItem(self.main)
        self.undoStack.push(cmd)
        cmd = ComponentEdit(self.main)
        cmd.redo()
        cmd = ComponentMerge(self.main)
        self.undoStack.push(cmd)
        cmdSA = ComponentSaveAs(self.main)
        cmdSA.redo()

        cmd = ComponentListChanged(self.main)
        cmd.directory = cmdSA.directory
        cmd.name = cmdSA.name
        self.undoStack.push(cmd)
        cmd = ComponentSave(self.main)
        cmd.redo()

    # save datasource item as action
    # \brief It saves the changes in the current datasource item with
    #        a new name
    def dsourceSaveAs(self):
        cmd = DataSourceEdit(self.main)
        cmd.redo()
        cmd = DataSourceApply(self.main)
        self.undoStack.push(cmd)
        cmdSA = DataSourceSaveAs(self.main)
        cmdSA.redo()

        cmd = DataSourceListChanged(self.main)
        cmd.directory = cmdSA.directory
        cmd.name = cmdSA.name
        self.undoStack.push(cmd)
        cmd = DataSourceSave(self.main)
        cmd.redo()

    # save all components item action
    # \brief It saves the changes in all components item
    def componentSaveAll(self):
        cmd = ComponentSaveAll(self.main)
        cmd.redo()
        self.undoStack.clear()

    # save all datasource item action
    # \brief It saves the changes in all datasources item
    def dsourceSaveAll(self):
        cmd = DataSourceSaveAll(self.main)
        cmd.redo()
        self.undoStack.clear()

    # change component directory action
    # \brief It changes the default component directory
    def componentChangeDirectory(self):
        cmd = ComponentChangeDirectory(self.main)
        cmd.redo()
        self.undoStack.clear()

    # change datasource directory action
    # \brief It changes the default datasource directory
    def dsourceChangeDirectory(self):
        cmd = DataSourceChangeDirectory(self.main)
        cmd.redo()
        self.undoStack.clear()

    # reload component list
    # \brief It changes the default component directory and reload components
    def componentReloadList(self):
        cmd = ComponentReloadList(self.main)
        cmd.redo()
        self.undoStack.clear()

    # reload datasource list
    # \brief It changes the default datasource directory and reload datasources
    def dsourceReloadList(self):
        cmd = DataSourceReloadList(self.main)
        cmd.redo()
        self.undoStack.clear()


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DataSource.py
# Data Source data class

""" Provides datasource widget data"""

import os
import copy
import sys

from PyQt5.QtCore import (QModelIndex, QFileInfo, QFile,
                          QIODevice, QTextStream)
from PyQt5.QtWidgets import (QApplication, QFileDialog, QMessageBox,
                             QWidget)
from PyQt5.QtXml import (QDomDocument)

from .NodeDlg import NodeDlg
from .DomTools import DomTools
from . import DataSourceDlg
from .DataSourceMethods import DataSourceMethods

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


# class with datasource variables
class Variables(object):
    pass


# dialog defining datasource
class CommonDataSource(object):

    # constructor
    def __init__(self, parent):

        # data source type
        self.dataSourceType = 'CLIENT'
        # attribute doc
        self.doc = u''

        # datasource dialog
        self.dialog = NodeDlg(parent)

        # datasource name
        self.dataSourceName = u''

        # external save method
        self.externalSave = None
        # external store method
        self.externalStore = None
        # external close method
        self.externalClose = None
        # external apply method
        self.externalApply = None

        # applied flag
        self.applied = False

        # datasource id
        self.id = None

        # if datasource in the component tree
        self.tree = False

        # datasource variables
        self.var = {}

        # creates variables dynamically
        self.clear()

    # clears the datasource content
    # \brief It sets the datasource variables to default values
    def clear(self):
        for ds, cl in DataSourceDlg.dsTypes.items():
            self.var[ds] = Variables()
            for vr in cl.var.keys():
                setattr(self.var[ds], vr, cl.var[vr])

    # provides the state of the datasource dialog
    # \returns state of the datasource in tuple
    def getState(self):
        state = [self.dataSourceType,
                 self.dataSourceName,
                 self.doc]

        for ds, cl in DataSourceDlg.dsTypes.items():
            for vr in cl.var.keys():
                vv = getattr(self.var[ds], vr)
                state.append(
                    copy.copy(vv)
                    if ((type(vv) is list) or (type(vv) is dict)) else vv)
        return tuple(state)

    # sets the state of the datasource dialog
    # \brief note that ids, applied and tree are not in the state
    # \param state state datasource written in tuple
    def setState(self, state):

        cnt = 3
        (self.dataSourceType, self.dataSourceName, self.doc) = state[:cnt]

        for ds, cl in DataSourceDlg.dsTypes.items():
            for vr in cl.var.keys():
                setattr(
                    self.var[ds], vr, copy.copy(state[cnt])
                    if ((type(state[cnt]) is list)
                        or (type(state[cnt]) is dict))
                    else state[cnt])
                cnt += 1


# dialog defining datasource
class DataSource(CommonDataSource):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(DataSource, self).__init__(parent)

        # dialog parent
        self.parent = parent

        # datasource dialog
        self.dialog = DataSourceDlg.CommonDataSourceDlg(self, parent)

        # datasource methods
        self.__methods = DataSourceMethods(self.dialog, self, self.parent)

        # datasource directory
        self.directory = ""

        # datasource file name
        self.name = None

        # DOM document
        self.document = None
        # saved XML
        self.savedXML = None

        # sub components
        self.components = []
        # sub datasources
        self.datasources = []

    # fetches $datasources and $components from xml
    # \brief populate components and datasources
    def fetchElements(self):
        dss = set()
        if not hasattr(self, "document"):
            return
        if hasattr(self.document, "toString"):
            xml = unicode(self.document.toString(0))
            dss = set(DomTools.findElements(xml, "datasources"))
        self.datasources = list(dss)

    # creates dialog
    # \brief It creates dialog, its GUI , updates Nodes and Form
    def createDialog(self):
        self.dialog = DataSourceDlg.CommonDataSourceDlg(self, self.parent)
        self.__methods.setDialog(self.dialog)
        self.createGUI()

        self.updateForm()
        self.updateNode()

    # clears the datasource content
    # \brief It sets the datasource variables to default values
    def clear(self):
        CommonDataSource.clear(self)

        if hasattr(self.dialog, "imp"):
            for ds in self.dialog.imp.values():
                if hasattr(ds, "clear"):
                    ds.clear()

    # checks if not saved
    # \returns True if it is not saved
    def isDirty(self):
        string = self.get()
        return False if string == self.savedXML else True

    # provides the datasource in xml string
    # \returns xml string
    def get(self, indent=0):
        if hasattr(self.document, "toString"):
            return unicode(self.document.toString(indent))

    # sets file name of the datasource and its directory
    # \param name name of the datasource
    # \param directory directory of the datasources
    def setName(self, name, directory):
        self.name = unicode(name)
        if not self.dataSourceName:
            # datasource name
            self.dataSourceName = self.name
            self.dialog.ui.nameLineEdit.setText(self.name)
            self.dialog.imp['CLIENT'].ui.cRecNameLineEdit.setText(self.name)
        if directory:
            self.directory = unicode(directory)

    # loads datasources from default file directory
    # \param fname optional file name
    def load(self, fname=None):
        if fname is None:
            if not self.name:
                filename = unicode(QFileDialog.getOpenFileName(
                    self.parent, "Open File", self.directory,
                    "XML files (*.xml);;HTML files (*.html);;"
                    "SVG files (*.svg);;User Interface files (*.ui)")[0])
                fi = QFileInfo(filename)
                fname = str(fi.fileName())
                if fname[-4:] == '.xml':
                    self.name = fname[:-4]
                    if self.name[-3:] == '.ds':
                        self.name = self.name[:-3]
                    else:
                        self.name = fname
            else:
                filename = os.path.join(self.directory, self.name + ".ds.xml")
        else:
            filename = fname
            if not self.name:
                fi = QFileInfo(filename)
                fname = str(fi.fileName())
                if fname[-4:] == '.xml':
                    self.name = fname[:-4]
                    if self.name[-3:] == '.ds':
                        self.name = self.name[:-3]
                    else:
                        self.name = fname
        try:
            fh = QFile(filename)
            if fh.open(QIODevice.ReadOnly):
                self.document = QDomDocument()
                self.root = self.document
#                if not self.document.setContent(self.repair(fh)):
                if not self.document.setContent(fh):
                    raise ValueError("could not parse XML")

                ds = DomTools.getFirstElement(self.document, "datasource")
                if ds:
                    self.setFromNode(ds)
                self.savedXML = self.document.toString(0)
            else:
                QMessageBox.warning(self.parent, "Cannot open the file",
                                    "Cannot open the file: %s" % (filename))
            try:
                self.createGUI()

            except Exception as e:
                QMessageBox.warning(self.parent, "dialog not created",
                                    "Problems in creating a dialog %s :\n\n%s"
                                    % (self.name, unicode(e)))

        except (IOError, OSError, ValueError) as e:
            error = "Failed to load: %s" % e
            logger.warn(error)
            QMessageBox.warning(self.parent, "Saving problem", error)

        except Exception as e:
            QMessageBox.warning(self.parent, "Saving problem", e)
            logger.warn(str(e))
        finally:
            if fh is not None:
                fh.close()
        self.fetchElements()
        if fh is not None:
            return filename

    # repairs xml datasources
    # \param xml xml string
    # \returns repaired xml
    def repair(self, xml):
        olddoc = QDomDocument()
        if not olddoc.setContent(xml):
            raise ValueError("could not parse XML")

        definition = olddoc.firstChildElement(str("definition"))
        if definition and definition.nodeName() == "definition":
            ds = definition.firstChildElement(str("datasource"))
            if ds and ds.nodeName() == "datasource":
                return xml

        ds = DomTools.getFirstElement(olddoc, "datasource")

        newdoc = QDomDocument()
        processing = newdoc.createProcessingInstruction("xml", "version='1.0'")
        newdoc.appendChild(processing)

        definition = newdoc.createElement(str("definition"))
        newdoc.appendChild(definition)

        if ds:
            newds = newdoc.importNode(ds, True)
        else:
            newds = newdoc.createElement(str("datasource"))
        definition.appendChild(newds)
        self.fetchElements()
        return newdoc.toString(0)

    # sets datasources from xml string
    # \param xml xml string
    # \param new True if datasource is not saved
    def set(self, xml, new=False):
        self.document = QDomDocument()
        self.root = self.document
        if not self.document.setContent(self.repair(xml)):
            raise ValueError("could not parse XML")
        else:
            if self.dialog and hasattr(self.dialog, "root"):
                self.dialog.root = self.document

        ds = DomTools.getFirstElement(self.document, "datasource")
        if ds:
            self.setFromNode(ds)
            if new:
                self.savedXML = ""
            else:
                self.savedXML = self.document.toString(0)
        try:
            self.createGUI()
        except Exception as e:
            QMessageBox.warning(self, "dialog not created",
                                "Problems in creating a dialog %s :\n\n%s"
                                % (self.name, unicode(e)))
        self.fetchElements()

    # accepts and save input text strings
    # \brief It copies the parameters and saves the dialog
    def save(self):

        error = None
        filename = os.path.join(self.directory, self.name + ".ds.xml")
        logger.info("saving in %s" % (filename))
        if filename:
            try:
                fh = QFile(filename)
                if not fh.open(QIODevice.WriteOnly):
                    raise IOError(unicode(fh.errorString()))
                stream = QTextStream(fh)
                self.createNodes()

                xml = self.repair(self.document.toString(0))
                document = QDomDocument()
                document.setContent(xml)
                stream << document.toString(2)
                self.savedXML = document.toString(0)
            except (IOError, OSError, ValueError) as e:
                error = "Failed to save: %s " % e \
                    + "Please try to use Save As command " \
                    + "or change the datasource directory"
                logger.warn(error)
                QMessageBox.warning(self.parent, "Saving problem", error)

            finally:
                if fh is not None:
                    fh.close()
        if not error:
            return True

    # provides the datasource name with its path
    # \returns datasource name with its path
    def getNewName(self):
        filename = unicode(
            QFileDialog.getSaveFileName(
                self.parent, "Save DataSource As ...", self.directory,
                "XML files (*.xml);;HTML files (*.html);;"
                "SVG files (*.svg);;User Interface files (*.ui)")[0])
        logger.info("saving in %s" % (filename))
        return filename

    # gets the current view
    # \returns the current view
    def __getview(self):
        if self.dialog and hasattr(self.parent, "view"):
            return self.dialog.view

    # sets the current view
    # \param view value to be set
    def __setview(self, view):
        if self.dialog and hasattr(self.dialog, "view"):
            self.dialog.view = view

    # attribute value
    view = property(__getview, __setview)

    # gets the current root
    # \returns the current root
    def __getroot(self):
        if self.dialog and hasattr(self.dialog, "root"):
            return self.dialog.root

    # sets the current root
    # \param root value to be set
    def __setroot(self, root):
        if self.dialog and hasattr(self.dialog, "root"):
            self.dialog.root = root

    # attribute value
    root = property(__getroot, __setroot)

    # shows dialog
    # \brief It adapts the dialog method
    def show(self):
        if hasattr(self, "datasource") and self.dialog:
            if self.dialog:
                return self.dialog.show()

    # clears the dialog
    # \brief clears the dialog
    def clearDialog(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.setDialog(None)

    # updates the form
    # \brief abstract class
    def updateForm(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.updateForm()

    # updates the node
    # \brief abstract class
    def updateNode(self, index=QModelIndex()):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            res = self.__methods.updateNode(index)
            self.fetchElements()
            return res

    # creates GUI
    # \brief abstract class
    def createGUI(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.createGUI()

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.setFromNode(node)

    # creates datasource node
    # \param external True if it should be create on a local DOM root,
    #        i.e. in component tree
    # \returns created DOM node
    def createNodes(self, external=False):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            res = self.__methods.createNodes(external)
            self.fetchElements()
            return res

    # accepts input text strings
    # \brief It copies the parameters and accept the dialog
    def apply(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            res = self.__methods.apply()
            self.fetchElements()
            return res

    # sets the tree mode used in ComponentDlg without save/close buttons
    # \param enable logical variable which dis-/enables mode
    def treeMode(self, enable=True):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.treeMode(enable)

    # connects the save action and stores the apply action
    # \param externalApply apply action
    # \param externalSave save action
    # \param externalClose close action
    # \param externalStore store action
    def connectExternalActions(self, externalApply=None, externalSave=None,
                               externalClose=None, externalStore=None):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.connectExternalActions(
                externalApply, externalSave, externalClose, externalStore)

    # reconnects save actions
    # \brief It reconnects the save action
    def reconnectSaveAction(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.reconnectSaveAction()

    # copies the datasource to the clipboard
    # \brief It copies the current datasource to the clipboard
    def copyToClipboard(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            return self.__methods.copyToClipboard()

    # copies the datasource from the clipboard to the current
    #  datasource dialog
    # \return status True on success
    def copyFromClipboard(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            res = self.__methods.copyFromClipboard()
            self.fetchElements()
            return res

    # creates the new empty header
    # \brief It clean the DOM tree and put into it xml and definition nodes
    def createHeader(self):
        if hasattr(self, "_DataSource__methods") and self.__methods:
            res = self.__methods.createHeader()
            self.fetchElements()
            return res


if __name__ == "__main__":
    import sys

    # Qt application
    app = QApplication(sys.argv)
    # data source form
    w = QWidget()
    w.show()
    # the first datasource form
    form = DataSource(w)

#    form.dataSourceType = 'CLIENT'
#    form.clientRecordName = 'counter1'
#    form.doc = 'The first client counter  '

#    form.dataSourceType = 'TANGO'
#    form.tangoDeviceName = 'p09/motor/exp.01'
#    form.tangoMemberName = 'Position'
#    form.tangoMemberType = 'attribute'
#    form.tangoHost = 'hasso.desy.de'
#    form.tangoPort = '10000'
#    form.tangoEncoding = 'LIMA2D'
#    form.tangoGroup = 'Coordinates'

#    form.dataSourceType = 'DB'
#    form.dbType = 'PGSQL'
#    form.dbDataFormat = 'SPECTRUM'
#    form.dbParameters = {'DB name':'tango', 'DB user':'tangouser'}

    form.createGUI()

    form.dialog.show()

    app.exec_()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file FileCommands.py
# user commands of GUI application

""" Component Designer commands """

from PyQt5.QtWidgets import (QMessageBox, QFileDialog, QUndoCommand,
                             QProgressDialog)
from PyQt5.QtCore import (Qt, QFileInfo)

from .DataSourceDlg import CommonDataSourceDlg
from . import DataSource
from .ComponentDlg import ComponentDlg
from .Component import Component
from .LabeledObject import LabeledObject

import sys
import logging
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


# Command which loads an existing component from the file
class ComponentOpen(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cpEdit = None
        self._cp = None
        self._fpath = None
        self._subwindow = None

    # executes the command
    # \brief It loads an existing component from the file
    def redo(self):
        if hasattr(self.receiver.ui, 'mdi'):
            if self._cp is None:
                self._cp = LabeledObject("", None)
            else:
                self._cp.instance = None

            self._cpEdit = Component(self.receiver.componentList)
            self._cpEdit.id = self._cp.id
            self._cpEdit.directory = self.receiver.componentList.directory
            self._cpEdit.createGUI()
            self._cpEdit.addContextMenu(self.receiver.contextMenuActions)
            if self._fpath:
                path = self._cpEdit.load(self._fpath)
            else:
                path = self._cpEdit.load()
                self._fpath = path

            if hasattr(self._cpEdit, "connectExternalActions"):
                self._cpEdit.connectExternalActions(
                    **self.receiver.externalCPActions)

            if path:
                self._cp.name = self._cpEdit.name
                self._cp.instance = self._cpEdit

                self.receiver.componentList.addElement(self._cp, False)
                self._cpEdit.dialog.setWindowTitle(
                    "%s [Component]" % self._cp.name)

                subwindow = self.receiver.subWindow(
                    self._cp.instance,
                    self.receiver.ui.mdi.subWindowList())
                if subwindow:
                    self.receiver.setActiveSubWindow(subwindow)
                    self._cp.instance.dialog.setSaveFocus()
                else:
                    self._subwindow = self.receiver.ui.mdi.addSubWindow(
                        self._cpEdit.dialog)
                    self._subwindow.resize(680, 560)
                    self._cpEdit.dialog.setSaveFocus()
                    self._cpEdit.dialog.show()
                    self._cp.instance = self._cpEdit
                self._cpEdit.dialog.show()
                logger.debug("EXEC componentOpen")

    # unexecutes the command
    # \brief It removes the loaded component from the component list
    def undo(self):
        if hasattr(self._cp, "instance"):
            if self._fpath:

                if hasattr(self._cp, 'instance') and self._cp.instance:
                    subwindow = self.receiver.subWindow(
                        self._cpEdit, self.receiver.ui.mdi.subWindowList())
                    if subwindow:
                        self.receiver.setActiveSubWindow(subwindow)
                        self.receiver.ui.mdi.closeActiveSubWindow()

                self.receiver.componentList.removeElement(
                    self._cp, False)
                self._cp.instance = None

        logger.debug("UNDO componentOpen")


# Command which loads an existing datasource from the file
class DataSourceOpen(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._dsEdit = None
        self._ds = None
        self._fpath = None
        self._subwindow = None

    # executes the command
    # \brief It loads an existing datasource from the file
    def redo(self):
        if hasattr(self.receiver.ui, 'mdi'):
            if self._ds is None:
                self._ds = LabeledObject("", None)
            else:
                self._ds.instance = None

            self._dsEdit = DataSource.DataSource(self.receiver.sourceList)
            self._dsEdit.id = self._ds.id
            self._dsEdit.directory = self.receiver.sourceList.directory
            if self._fpath:
                path = self._dsEdit.load(self._fpath)
            else:
                path = self._dsEdit.load()
                self._fpath = path
            if hasattr(self._dsEdit, "connectExternalActions"):
                self._dsEdit.connectExternalActions(
                    **self.receiver.externalDSActions)
            if path:
                self._ds.name = self._dsEdit.name
                self._ds.instance = self._dsEdit

                self.receiver.sourceList.addElement(self._ds, False)

                self._dsEdit.dialog.setWindowTitle(
                    "%s [DataSource]" % self._ds.name)

                subwindow = self.receiver.subWindow(
                    self._ds.instance,
                    self.receiver.ui.mdi.subWindowList())
                if subwindow:
                    self.receiver.setActiveSubWindow(subwindow)
                    self._ds.instance.dialog.setSaveFocus()
                else:
                    self._subwindow = self.receiver.ui.mdi.addSubWindow(
                        self._dsEdit.dialog)
                    self._subwindow.resize(440, 550)
                    self._dsEdit.dialog.setSaveFocus()
                    self._dsEdit.dialog.show()
                    self._ds.instance = self._dsEdit

                self._dsEdit.dialog.show()
                logger.debug("EXEC dsourceOpen")

    # unexecutes the command
    # \brief It removes the loaded datasource from the datasource list
    def undo(self):
        if hasattr(self._ds, "instance"):
            if self._fpath:

                if hasattr(self._ds, 'instance'):
                    subwindow = self.receiver.subWindow(
                        self._ds.instance,
                        self.receiver.ui.mdi.subWindowList())
                    if subwindow:
                        self.receiver.setActiveSubWindow(subwindow)
                        self.receiver.ui.mdi.closeActiveSubWindow()

                self.receiver.sourceList.removeElement(self._ds, False)
                self._ds.instance = None

        logger.debug("UNDO dsourceOpen")


# Command which saves with the current component in the file
class ComponentSave(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None
        self._cpEdit = None
        self._subwindow = None

    # executes the command
    # \brief It saves with the current component in the file
    def redo(self):
        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is None:
            QMessageBox.warning(
                self.receiver, "Component not selected",
                "Please select one of the components")
        else:
            if self._cp.instance is None:
                #                self._cpEdit = FieldWg()
                self._cpEdit = Component(self.receiver.componentList)
                self._cpEdit.id = self._cp.id
                self._cpEdit.directory = \
                    self.receiver.componentList.directory
                self._cpEdit.name = self.receiver.componentList.elements[
                    self._cp.id].name
                self._cpEdit.createGUI()
                self._cpEdit.addContextMenu(
                    self.receiver.contextMenuActions)
                self._cpEdit.createHeader()
                self._cpEdit.dialog.setWindowTitle(
                    "%s [Component]" % self._cp.name)
            else:
                self._cpEdit = self._cp.instance

            if hasattr(self._cpEdit, "connectExternalActions"):
                self._cpEdit.connectExternalActions(
                    **self.receiver.externalCPActions)

            subwindow = self.receiver.subWindow(
                self._cpEdit, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
            else:
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._cpEdit.dialog)
                self._subwindow.resize(680, 560)
                self._cpEdit.dialog.show()
            self._cp.instance = self._cpEdit

            if self._cpEdit.save():
                self._cp.savedName = self._cp.name
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()

        logger.debug("EXEC componentSave")

    # unexecutes the command
    # \brief It populates the component list
    def undo(self):
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()
        logger.debug("UNDO componentSave")


# Command which saves all components in the file
class ComponentSaveAll(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It saves all components in the file
    def redo(self):

        keys = self.receiver.componentList.elements.keys()
        progress = QProgressDialog(
            "Saving Component elements",
            "", 0, len(keys), self.receiver.componentList)
        progress.setWindowTitle("Save All Components")
        progress.setWindowModality(Qt.WindowModal)
        progress.setCancelButton(None)
        progress.show()
        for i in range(len(keys)):
            icp = keys[i]
            cp = self.receiver.componentList.elements[icp]
            if cp.instance is None:
                #                self._cpEdit = FieldWg()
                cpEdit = Component(self.receiver.componentList)
                cpEdit.id = cp.id
                cpEdit.directory = self.receiver.componentList.directory
                cpEdit.name = \
                    self.receiver.componentList.elements[cp.id].name
                cpEdit.createGUI()
                cpEdit.addContextMenu(self.receiver.contextMenuActions)
                cpEdit.createHeader()
                cpEdit.dialog.setWindowTitle("%s [Component]" % cp.name)
                cp.instance = cpEdit

            if cp.instance is not None:
                cp.instance.merge(False)
                cp.instance.save()
            progress.setValue(i)
        progress.setValue(len(keys))
        progress.close()

        logger.debug("EXEC componentSaveAll")


# Command which saves the current components in the file with a different name
class ComponentSaveAs(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        # new name of component
        self.name = None
        # directory of the component file
        self.directory = None

        self._cp = None
        self._pathFile = None
        self._subwindow = None

    # executes the command
    # \brief It saves the current components in the file with a different name
    def redo(self):
        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is None:
            QMessageBox.warning(self.receiver, "Component not selected",
                                "Please select one of the components")
        else:
            if self._cp.instance is not None:
                self._pathFile = self._cp.instance.getNewName()
                fi = QFileInfo(self._pathFile)
                self.name = unicode(fi.fileName())
                if self.name[-4:] == '.xml':
                    self.name = self.name[:-4]
                self.directory = unicode(fi.dir().path())
                self._cp.instance.name = self.name
                self._cp.instance.directory = self.directory

        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()
        logger.debug("EXEC componentSaveAs")

    # unexecutes the command
    # \brief It populates the Component list
    def undo(self):
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()
        logger.debug("UNDO componentSaveAs")


# Command which changes the current component file directory
class ComponentChangeDirectory(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It changes the current component file directory
    def redo(self):
        failures = []
        if not self.receiver.closeList(
                None, self.receiver.componentList, failures):
            return
        if (failures and QMessageBox.question(
                self.receiver,
                "Component - Reload List from Configuration server",
                ("All unsaved components will be lost. "
                 "Would you like to proceed ?").encode(),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes) == QMessageBox.No):
            return

        path = unicode(QFileDialog.getExistingDirectory(
            self.receiver, "Open Directory",
            self.receiver.componentList.directory,
            QFileDialog.ShowDirsOnly or QFileDialog.DontResolveSymlinks))

        if not path:
            return
        subwindows = self.receiver.ui.mdi.subWindowList()
        if subwindows:
            for subwindow in subwindows:
                if isinstance(subwindow.widget(), ComponentDlg):
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.componentList.elements = {}
        self.receiver.componentList.directory = path
        self.receiver.updateStatusBar()

        self.receiver.loadComponents()

        logger.debug("EXEC componentChangeDirectory")


# Command which saves all the datasources in files
class DataSourceSaveAll(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It saves all the datasources in files
    def redo(self):
        keys = self.receiver.sourceList.elements.keys()
        progress = QProgressDialog(
            "Saving DataSource elements",
            "", 0, len(keys), self.receiver.sourceList)
        progress.setWindowTitle("Save All DataSources")
        progress.setWindowModality(Qt.WindowModal)
        progress.setCancelButton(None)
        progress.show()

        for i in range(len(keys)):
            ids = keys[i]
            ds = self.receiver.sourceList.elements[ids]
            if ds.instance is None:
                dsEdit = DataSource.DataSource(self.receiver.sourceList)
                dsEdit.id = ds.id
                dsEdit.directory = self.receiver.sourceList.directory
                dsEdit.name = \
                    self.receiver.sourceList.elements[ds.id].name
                ds.instance = dsEdit

            if ds.instance is not None:
                if ds.instance.save():
                    ds.savedName = ds.name
            progress.setValue(i)
        progress.setValue(len(keys))
        progress.close()

        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("EXEC dsourceSaveAll")


# Command which saves the current datasource in files
class DataSourceSave(QUndoCommand):
    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None

    # executes the command
    # \brief It saves the current datasource in files
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(
                self.receiver, "DataSource not selected",
                "Please select one of the datasources")

        if self._ds is not None and hasattr(self._ds, "instance"):
            if self._ds.instance is None:
                dsEdit = DataSource.DataSource(self.receiver.sourceList)
                dsEdit.id = self._ds.id
                dsEdit.directory = self.receiver.sourceList.directory
                dsEdit.name = self.receiver.sourceList.elements[
                    self._ds.id].name
                self._ds.instance = dsEdit

            if self._ds.instance.save():
                self._ds.savedName = self._ds.name

        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("EXEC dsourceSave")

    # unexecutes the command
    # \brief It populates the datasource list
    def undo(self):
        ds = self.receiver.sourceList.currentListElement()
        if hasattr(ds, "id"):
            self.receiver.sourceList.populateElements(ds.id)
        else:
            self.receiver.sourceList.populateElements()
        logger.debug("UNDO dsourceSave")


# Command which saves the current datasource in files with a different name
class DataSourceSaveAs(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        # new datasource name
        self.name = None
        # new file directory
        self.directory = None

        self._ds = None
        self._pathFile = None

    # executes the command
    # \brief It saves the current datasource in files with a different name
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(self.receiver, "DataSource not selected",
                                "Please select one of the datasources")
        else:
            if self._ds.instance is None:
                dsEdit = DataSource.DataSource(self.receiver.sourceList)
                dsEdit.id = self._ds.id
                dsEdit.directory = self.receiver.sourceList.directory
                dsEdit.name = self.receiver.sourceList.elements[
                    self._ds.id].name
                self._ds.instance = dsEdit

            if self._ds.instance is not None:
                self._pathFile = self._ds.instance.getNewName()
                fi = QFileInfo(self._pathFile)
                self.name = unicode(fi.fileName())
                if self.name[-4:] == '.xml':
                    self.name = self.name[:-4]
                    if self.name[-3:] == '.ds':
                        self.name = self.name[:-3]

                self.directory = unicode(fi.dir().path())

        logger.debug("EXEC dsourceSaveAs")


# Command which changes the current file directory with datasources
class DataSourceChangeDirectory(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It changes the current file directory with datasources
    def redo(self):
        failures = []
        if not self.receiver.closeList(
                None, self.receiver.sourceList, failures):
            return
        if (failures and QMessageBox.question(
                self.receiver,
                "DataSource - Reload List from Configuration Server",
                ("All unsaved datasources will be lost. "
                 "Would you like to proceed ?").encode(),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes) == QMessageBox.No):
            return

        path = unicode(QFileDialog.getExistingDirectory(
            self.receiver, "Open Directory",
            self.receiver.sourceList.directory,
            QFileDialog.ShowDirsOnly or QFileDialog.DontResolveSymlinks))

        if not path:
            return

        subwindows = self.receiver.ui.mdi.subWindowList()
        if subwindows:
            for subwindow in subwindows:
                if isinstance(subwindow.widget(), CommonDataSourceDlg):
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.sourceList.elements = {}
        self.receiver.sourceList.directory = path
        self.receiver.updateStatusBar()

        self.receiver.loadDataSources()

        logger.debug("EXEC dsourceChangeDirectory")


# Command which reloads the components from the current component directory
#  into the component list
class ComponentReloadList(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It reloads the components from the current component directory
    #        into the component list
    def redo(self):
        failures = []
        if not self.receiver.closeList(
                None, self.receiver.componentList, failures):
            return
        if (failures and QMessageBox.question(
                self.receiver,
                "Component - Reload List from Configuration server",
                ("All unsaved components will be lost. "
                 "Would you like to proceed ?").encode(),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes) == QMessageBox.No):
            return

        subwindows = self.receiver.ui.mdi.subWindowList()
        if subwindows:
            for subwindow in subwindows:
                if isinstance(subwindow.widget(), ComponentDlg):
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.componentList.elements = {}
        self.receiver.loadComponents()

        logger.debug("EXEC componentReloadList")


# Command which reloads the datasources from the current datasource directory
#  into the datasource list
class DataSourceReloadList(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver

    # executes the command
    # \brief It reloads the datasources from the current datasource directory
    #        into the datasource list
    def redo(self):
        failures = []
        if not self.receiver.closeList(
                None, self.receiver.sourceList, failures):
            return
        if (failures and QMessageBox.question(
                self.receiver,
                "DataSource - Reload List from Configuration Server",
                ("All unsaved datasources will be lost. "
                 "Would you like to proceed ?").encode(),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes) == QMessageBox.No):
            return

        subwindows = self.receiver.ui.mdi.subWindowList()
        if subwindows:
            for subwindow in subwindows:
                if isinstance(subwindow.widget(), CommonDataSourceDlg):
                    self.receiver.setActiveSubWindow(subwindow)
                    self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.sourceList.elements = {}
        self.receiver.loadDataSources()

        logger.debug("EXEC dsourceReloadList")


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ComponentDlg.py
# component classes

""" component widget """

from PyQt5.QtWidgets import (QDialog, QWidget)
from PyQt5 import uic

# from .ui.ui_componentdlg import Ui_ComponentDlg
from .DomTools import DomTools

import logging
import os
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "componentdlg.ui"))


# compoent dialog
class ComponentDlg(QDialog):

    # constructor
    # \param component component instance
    # \param parent patent instance
    def __init__(self, component, parent=None):
        super(ComponentDlg, self).__init__(parent)
        # component instance
        self.component = component

        # user interface
        self.ui = _formclass()

        # default widget
        self.ui.widget = QWidget(self)

    # provides row number of the given node
    # \param child child item
    # \returns row number
    def getWidgetNodeRow(self, child):
        if self.ui and self.ui.widget:
            return DomTools.getNodeRow(child, self.ui.widget.node)
        else:
            logger.warn("Widget does not exist")

    # sets focus on save button
    # \brief It sets focus on save button
    def setSaveFocus(self):
        if self.ui:
            self.ui.savePushButton.setFocus()

    # closes the window and cleans the dialog label
    # \param event closing event
    def closeEvent(self, event):
        super(ComponentDlg, self).closeEvent(event)
        self.component.dialog = None
        event.accept()

    # rejects component closing
    def reject(self):
        self.parent().close()
        super(ComponentDlg, self).reject()


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file EditCommands.py
# user commands of GUI application

""" Component Designer commands """

from PyQt5.QtWidgets import (QUndoCommand, QMessageBox)

from .DataSourceDlg import DataSourceDlg
from . import DataSource
from .Component import Component

import sys
import logging
# message logger
logger = logging.getLogger("nxsdesigner")


def iternext(it):
    if sys.version_info > (3,):
        return next(iter(it.values()))
    else:
        return it.itervalues().next()


# Command which opens dialog with the current component
class ComponentEdit(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None
        self._cpEdit = None
        self._subwindow = None

    # executes the command
    # \brief It opens dialog with the current component
    def redo(self):
        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is None:
            QMessageBox.warning(self.receiver, "Component not selected",
                                "Please select one of the components")
        else:
            if self._cp.instance is None:
                #                self._cpEdit = FieldWg()
                self._cpEdit = Component(self.receiver.componentList)
                self._cpEdit.id = self._cp.id
                self._cpEdit.directory = \
                    self.receiver.componentList.directory
                self._cpEdit.name = self.receiver.componentList.elements[
                    self._cp.id].name
                self._cpEdit.createGUI()
                self._cpEdit.addContextMenu(
                    self.receiver.contextMenuActions)
                self._cpEdit.createHeader()
                self._cpEdit.dialog.setWindowTitle(
                    "%s [Component]*" % self._cp.name)
            else:
                self._cpEdit = self._cp.instance

            if hasattr(self._cpEdit, "connectExternalActions"):
                self._cpEdit.connectExternalActions(
                    **self.receiver.externalCPActions)

            subwindow = self.receiver.subWindow(
                self._cpEdit, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._cpEdit.reconnectSaveAction()
            else:
                self._cpEdit.createGUI()

                self._cpEdit.addContextMenu(
                    self.receiver.contextMenuActions)
                if self._cpEdit.isDirty():
                    self._cpEdit.dialog.setWindowTitle(
                        "%s [Component]*" % self._cp.name)
                else:
                    self._cpEdit.dialog.setWindowTitle(
                        "%s [Component]" % self._cp.name)

                self._cpEdit.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._cpEdit.dialog)
                self._subwindow.resize(680, 560)
                self._cpEdit.dialog.show()
            self._cp.instance = self._cpEdit

        logger.debug("EXEC componentEdit")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO componentEdit")


# Command which copies the current datasource into the clipboard
class DataSourceCopy(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None
        self._oldstate = None
        self._newstate = None
        self._subwindow = None

    # executes the command
    # \brief It copies the current datasource into the clipboard
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(self.receiver, "DataSource not selected",
                                "Please select one of the datasources")
        if self._ds is not None and self._ds.instance is not None:
            if self._newstate is None:
                if self._oldstate is None:
                    self._oldstate = self._ds.instance.getState()
                self._ds.instance.copyToClipboard()
            else:
                self.receiver.sourceList.elements[
                    self._ds.id].instance.setState(self._newstate)
                self._ds.instance.updateForm()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createGUI()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

            self._newstate = self._ds.instance.getState()

        logger.debug("EXEC dsourceCopy")

    # unexecutes the command
    # \brief It updates state of datasource to the old state
    def undo(self):
        if self._ds is not None and hasattr(self._ds, 'instance') \
                and self._ds.instance is not None:

            self.receiver.sourceList.elements[
                self._ds.id].instance.setState(self._oldstate)
            self.receiver.sourceList.elements[
                self._ds.id].instance.updateForm()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createGUI()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

        logger.debug("UNDO dsourceCopy")


# Command which moves the current datasource into the clipboard
class DataSourceCut(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None
        self._oldstate = None
        self._newstate = None
        self._subwindow = None

    # executes the command
    # \brief It moves the current datasource into the clipboard
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(self.receiver, "DataSource not selected",
                                "Please select one of the datasources")
        if self._ds is not None and self._ds.instance is not None:
            if self._newstate is None:
                if self._oldstate is None:
                    self._oldstate = self._ds.instance.getState()
                self._ds.instance.copyToClipboard()
                self._ds.instance.clear()
                self._ds.instance.updateForm()
                self._ds.instance.dialog.show()
            else:
                self.receiver.sourceList.elements[
                    self._ds.id].instance.setState(self._newstate)
                self._ds.instance.updateForm()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createGUI()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

            self._newstate = self._ds.instance.getState()
        if hasattr(self._ds, "id"):
            self.receiver.sourceList.populateElements(self._ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("EXEC dsourceCut")

    # unexecutes the command
    # \brief It copy back the removed datasource
    def undo(self):
        if self._ds is not None and hasattr(self._ds, 'instance') \
                and self._ds.instance is not None:

            self.receiver.sourceList.elements[
                self._ds.id].instance.setState(self._oldstate)
            self.receiver.sourceList.elements[
                self._ds.id].instance.updateForm()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createGUI()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

        if hasattr(self._ds, "id"):
            self.receiver.sourceList.populateElements(self._ds.id)
        else:
            self.receiver.sourceList.populateElements()

        logger.debug("UNDO dsourceCut")


# Command which pastes the current datasource from the clipboard
class DataSourcePaste(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None
        self._oldstate = None
        self._newstate = None
        self._subwindow = None

    # executes the command
    # \brief It pastes the current datasource from the clipboard
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(self.receiver, "DataSource not selected",
                                "Please select one of the datasources")
        if self._ds is not None and self._ds.instance is not None:
            if self._newstate is None:
                if self._oldstate is None:
                    self._oldstate = self._ds.instance.getState()
                self._ds.instance.clear()
                if not self._ds.instance.copyFromClipboard():
                    QMessageBox.warning(
                        self.receiver, "Pasting item not possible",
                        "Probably clipboard does not contain datasource")

                self._ds.instance.updateForm()
#                self._ds.instance.updateNode()
                self._ds.instance.dialog.setFrames(
                    self._ds.instance.dataSourceType)

#                self._ds.instance.updateForm()
                self._ds.instance.dialog.show()
            else:
                self.receiver.sourceList.elements[
                    self._ds.id].instance.setState(self._newstate)
                self._ds.instance.updateForm()
#                self._ds.instance.updateNode()

            self._newstate = self._ds.instance.getState()

            self.receiver.sourceList.elements[
                self._ds.id].instance.setState(self._oldstate)
            self._ds.instance.updateNode()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createDialog()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

            if hasattr(self._ds, "id"):
                self.receiver.sourceList.populateElements(self._ds.id)
            else:
                self.receiver.sourceList.populateElements()
        logger.debug("EXEC dsourcePaste")

    # unexecutes the command
    # \brief It remove the pasted datasource
    def undo(self):
        if self._ds is not None and hasattr(self._ds, 'instance') \
                and self._ds.instance is not None:

            self.receiver.sourceList.elements[
                self._ds.id].instance.setState(self._oldstate)
            self.receiver.sourceList.elements[
                self._ds.id].instance.updateForm()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createGUI()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

            if hasattr(self._ds, "id"):
                self.receiver.sourceList.populateElements(self._ds.id)
            else:
                self.receiver.sourceList.populateElements()

        logger.debug("UNDO dsourcePaste")


# Command which applies the changes from the form for the current datasource
class DataSourceApply(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None
        self._oldstate = None
        self._newstate = None
        self._subwindow = None

    # executes the command
    # \brief It applies the changes from the form for the current datasource
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(self.receiver, "DataSource not selected",
                                "Please select one of the datasources")
            return
        if self._ds.instance is None:
            #                self._dsEdit = FieldWg()
            self._ds.instance = DataSource.DataSource(self.receiver.sourceList)
            self._ds.instance.id = self._ds.id
            self._ds.instance.directory = \
                self.receiver.sourceList.directory
            self._ds.instance.name = self.receiver.sourceList.elements[
                self._ds.id].name
        if not self._ds.instance.dialog:
            self._ds.instance.createDialog()
            self._ds.instance.dialog.setWindowTitle(
                "%s [DataSource]*" % self._ds.name)

            if hasattr(self._ds.instance, "connectExternalActions"):
                self._ds.instance.connectExternalActions(
                    **self.receiver.externalDSActions)
            self._subwindow = self.receiver.ui.mdi.addSubWindow(
                self._ds.instance.dialog)
            self._subwindow.resize(440, 550)
            self._ds.instance.dialog.show()

        if self._ds is not None and self._ds.instance is not None:
            if self._newstate is None:
                if self._oldstate is None:
                    self._oldstate = self._ds.instance.getState()
            else:
                self.receiver.sourceList.elements[
                    self._ds.id].instance.setState(
                    self._newstate)
                if not hasattr(self._ds.instance.dialog.ui, "docTextEdit"):
                    self._ds.instance.createDialog()
                self._ds.instance.updateForm()

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                self._ds.instance.createGUI()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [Component]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [Component]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._ds.instance.dialog.show()

            self._ds.instance.apply()
            self._newstate = self._ds.instance.getState()

            if hasattr(self._ds, "id"):
                self.receiver.sourceList.populateElements(self._ds.id)
            else:
                self.receiver.sourceList.populateElements()
        else:
            QMessageBox.warning(self.receiver, "DataSource not created",
                                "Please edit one of the datasources")

        logger.debug("EXEC dsourceApply")

    # unexecutes the command
    # \brief It recovers the old state of the current datasource
    def undo(self):
        if self._ds is not None and hasattr(self._ds, 'instance') \
                and self._ds.instance is not None:

            self.receiver.sourceList.elements[
                self._ds.id].instance.setState(self._oldstate)

            subwindow = self.receiver.subWindow(
                self._ds.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self.receiver.sourceList.elements[
                    self._ds.id].instance.updateForm()
                self._ds.instance.reconnectSaveAction()

            else:
                self._ds.instance.createDialog()

                self.receiver.sourceList.elements[
                    self._ds.id].instance.updateForm()

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)

            self._ds.instance.updateNode()
            if self._ds.instance.isDirty():
                self._ds.instance.dialog.setWindowTitle(
                    "%s [Component]*" % self._ds.name)
            else:
                self._ds.instance.dialog.setWindowTitle(
                    "%s [Component]" % self._ds.name)
            self._ds.instance.dialog.show()

            if hasattr(self._ds, "id"):
                self.receiver.sourceList.populateElements(self._ds.id)
            else:
                self.receiver.sourceList.populateElements()

        logger.debug("UNDO dsourceApply")


# Command which takes the datasources from the current component
class ComponentTakeDataSources(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None

    # executes the command
    # \brief It reloads the datasources from the current datasource directory
    #        into the datasource list
    def redo(self):

        if self._cp is None:
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is None:
            QMessageBox.warning(self.receiver, "Component not selected",
                                "Please select one of the components")
        else:
            if self._cp.instance is not None:
                datasources = self._cp.instance.getDataSources()

                if datasources:
                    dialogs = self.receiver.ui.mdi.subWindowList()
                    if dialogs:
                        for dialog in dialogs:
                            if isinstance(dialog, DataSourceDlg):
                                self.receiver.setActiveSubWindow(
                                    dialog)
                                self.receiver.ui.mdi.closeActiveSubWindow()

                    self.receiver.setDataSources(datasources, new=True)
                else:
                    QMessageBox.warning(
                        self.receiver, "DataSource item not selected",
                        "Please select one of the datasource items")

        logger.debug("EXEC componentTakeDataSources")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO componentTakeDataSources")


# Command which takes the datasources from the current component
class ComponentTakeDataSource(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None
        self._ids = None
        self._ds = None
        self._lids = None

    # executes the command
    # \brief It reloads the datasources from the current datasource directory
    #        into the datasource list
    def redo(self):

        if not self._lids:
            self._lids = \
                iternext(self.receiver.sourceList.elements).id \
                if len(self.receiver.sourceList.elements) else None
        if self._ids and self._ds:
            self.receiver.sourceList.addElement(self._ds)
            self.receiver.sourceList.populateElements(self._ids)

        else:
            if self._cp is None:
                self._cp = \
                    self.receiver.componentList.currentListElement()
            if self._cp is not None:
                if self._cp.instance is not None:

                    datasource = self._cp.instance.getCurrentDataSource()
                    if datasource:
                        dialogs = self.receiver.ui.mdi.subWindowList()
                        if dialogs:
                            for dialog in dialogs:
                                if isinstance(dialog, DataSourceDlg):
                                    self.receiver.ui.mdi.\
                                        setActiveSubWindow(
                                            dialog)
                                    self.receiver.ui.mdi\
                                        .closeActiveSubWindow()

                        self._ids = self.receiver.setDataSources(
                            datasource, new=True)
                        self._ds = \
                            self.receiver.sourceList.elements[self._ids]
                        self.receiver.sourceList.populateElements(
                            self._ids)
                    else:
                        QMessageBox.warning(
                            self.receiver, "DataSource item not selected",
                            "Please select one of the datasource items")

        logger.debug("EXEC componentTakeDataSource")

    # unexecutes the command
    # \brief It does nothing
    def undo(self):
        logger.debug("UNDO componentTakeDataSource")

        self.receiver.sourceList.removeElement(self._ds, False)
        if hasattr(self._ds, 'instance'):
            subwindow = self.receiver.subWindow(
                self._ds.instance,
                self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self.receiver.ui.mdi.closeActiveSubWindow()

        self.receiver.sourceList.populateElements(self._lids)


# Command which opens the dialog with the current datasource
class DataSourceEdit(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._ds = None
        self._dsEdit = None
        self._subwindow = None

    # executes the command
    # \brief It opens the dialog with the current datasource
    def redo(self):
        if self._ds is None:
            self._ds = self.receiver.sourceList.currentListElement()
        if self._ds is None:
            QMessageBox.warning(self.receiver, "DataSource not selected",
                                "Please select one of the datasources")
        else:
            if self._ds.instance is None:
                #                self._dsEdit = FieldWg()
                self._dsEdit = DataSource.DataSource(self.receiver.sourceList)

                self._dsEdit.id = self._ds.id
                self._dsEdit.directory = self.receiver.sourceList.directory
                self._dsEdit.name = self.receiver.sourceList.elements[
                    self._ds.id].name
                self._dsEdit.createDialog()
                self._dsEdit.dialog.setWindowTitle(
                    "%s [DataSource]*" % self._ds.name)
                self._ds.instance = self._dsEdit
            else:
                if not self._ds.instance.dialog:
                    self._ds.instance.createDialog()
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                self._dsEdit = self._ds.instance

            if hasattr(self._dsEdit, "connectExternalActions"):
                self._dsEdit.connectExternalActions(
                    **self.receiver.externalDSActions)

            subwindow = self.receiver.subWindow(
                self._dsEdit, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
                self._ds.instance.reconnectSaveAction()
            else:
                if self._ds.instance.dialog is None:
                    self._ds.instance.createDialog()

                if self._ds.instance.isDirty():
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]*" % self._ds.name)
                else:
                    self._ds.instance.dialog.setWindowTitle(
                        "%s [DataSource]" % self._ds.name)

                self._ds.instance.reconnectSaveAction()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._ds.instance.dialog)
                self._subwindow.resize(440, 550)
                self._dsEdit.dialog.show()

        logger.debug("EXEC dsourceEdit")


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ElementList.py
# Data source list class

""" datasource list widget """
import os
import sys

from PyQt5.QtCore import (Qt, )
from PyQt5.QtWidgets import (QWidget, QMenu, QMessageBox, QListWidgetItem,
                             QProgressDialog)
from PyQt5 import uic

# from .ui.ui_elementlist import Ui_ElementList
from .LabeledObject import LabeledObject


import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "elementlist.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a group tag
class ElementList(QWidget):

    # constructor
    # \param directory datasource directory
    # \param parent patent instance
    def __init__(self, directory, parent=None):
        super(ElementList, self).__init__(parent)
        # directory from which components are loaded by default
        self.directory = directory

        # group elements
        self.elements = {}

        # actions
        self._actions = []

        # user interface
        self.ui = _formclass()

        # widget title
        self.title = "Elements"
        # singular name
        self.clName = "Element"
        # class name
        self.name = "elements"
        # extention
        self.extention = ".xml"
        # excluded extention
        self.disextention = None

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):

        self.ui.setupUi(self)
        self.ui.elementTabWidget.setTabText(0, self.title)
        self.ui.elementListWidget.setEditTriggers(
            self.ui.elementListWidget.SelectedClicked)

        self.populateElements()

    # opens context Menu
    # \param position in the element list
    def _openMenu(self, position):
        menu = QMenu()
        for action in self._actions:
            if action is None:
                menu.addSeparator()
            elif isinstance(action, dict):
                for k in action:
                    submenu = menu.addMenu(k)
                    for saction in action[k]:
                        if saction is None:
                            submenu.addSeparator()
                        else:
                            submenu.addAction(saction)
            else:
                menu.addAction(action)
        menu.exec_(self.ui.elementListWidget.viewport().mapToGlobal(position))

    # sets context menu actions for the element list
    # \param actions tuple with actions
    def setActions(self, actions):
        self.ui.elementListWidget.setContextMenuPolicy(Qt.CustomContextMenu)
        self.ui.elementListWidget.customContextMenuRequested.connect(
            self._openMenu)
        self._actions = actions

    # adds an element
    #  \brief It runs the Element Dialog and fetches element
    #         name and value
    def addElement(self, obj, flag=True):
        self.elements[obj.id] = obj
        self.populateElements(obj.id, flag)

    # takes a name of the current element
    # \returns name of the current element
    def currentListElement(self):
        item = self.ui.elementListWidget.currentItem()
        if item is not None \
           and item.data(Qt.UserRole) \
           in self.elements.keys():
            return self.elements[item.data(Qt.UserRole)]
        else:
            return None

    # sets focus into element list
    def setItemFocus(self):
        self.ui.elementListWidget.setFocus()

    # removes the current element
    #  \brief It removes the current element asking before about it
    def removeElement(self, obj=None, question=True):

        if obj is not None:
            oid = obj.id
        else:
            cds = self.currentListElement()
            if cds is None:
                return
            oid = cds.id
        if oid is None:
            return
        if oid in self.elements.keys():
            if question:
                if QMessageBox.question(
                    self, "%s - Close" % self.clName,
                    "Close %s: %s ".encode()
                    % (self.clName, self.elements[oid].name),
                        QMessageBox.Yes | QMessageBox.No,
                        QMessageBox.Yes) == QMessageBox.No:
                    return

            self.elements.pop(oid)
            self.populateElements()

    # changes the current value of the element
    # \brief It changes the current value of the element and informs
    #        the user that element names arenot editable
    def listItemChanged(self, item, name=None):
        cle = self.currentListElement()
        if cle:
            ide = cle.id
            if ide in self.elements.keys():
                old = self.elements[ide]
                oname = self.elements[ide].name
                if name is None:
                    self.elements[ide].name = unicode(item.text())
                else:
                    self.elements[ide].name = name
                self.populateElements()
                return old, oname
        return None, None

    # fills in the element list
    # \param selectedElement selected element
    # \param edit flag if edit the selected item
    def populateElements(self, selectedElement=None, edit=False):
        selected = None
        self.ui.elementListWidget.clear()

        slist = [(self.elements[key].name, key)
                 for key in self.elements.keys()]
        slist.sort()

        for name, el in slist:
            item = QListWidgetItem(str("%s" % name))
            item.setData(Qt.UserRole, (self.elements[el].id))
            item.setFlags(item.flags() | Qt.ItemIsEditable)
            dirty = False
            if hasattr(self.elements[el], "isDirty") \
                    and self.elements[el].isDirty():
                dirty = True
            if self.elements[el].instance is not None:
                if hasattr(self.elements[el].instance, "isDirty") \
                        and self.elements[el].instance.isDirty():
                    dirty = True
            if dirty:
                item.setForeground(Qt.red)
            else:
                item.setForeground(Qt.black)

            self.ui.elementListWidget.addItem(item)
            if selectedElement is not None \
                    and selectedElement == self.elements[el].id:
                selected = item

            if self.elements[el].instance is not None \
                    and self.elements[el].instance.dialog is not None:
                try:
                    if dirty:
                        self.elements[el].instance.dialog.\
                            setWindowTitle("%s [%s]*" % (name, self.clName))
                    else:
                        self.elements[el].instance.dialog.\
                            setWindowTitle("%s [%s]" % (name, self.clName))
                except Exception:
                    self.elements[el].instance.dialog = None

        if selected is not None:
            selected.setSelected(True)
            self.ui.elementListWidget.setCurrentItem(selected)
            if edit:
                self.ui.elementListWidget.editItem(selected)

    # sets the elements
    # \param elements dictionary with the elements, i.e. name:xml
    # \param externalActions dictionary with external actions
    # \param itemActions actions of the context menu
    # \param new logical variableset to True if element is not saved
    def setList(self, elements, externalActions=None,
                itemActions=None, new=False):
        if not os.path.isdir(self.directory):
            try:
                if os.path.exists(os.path.join(os.getcwd(), self.name)):
                    self.directory = os.path.abspath(
                        os.path.join(os.getcwd(), self.name))
                else:
                    self.directory = os.getcwd()
            except Exception:
                return

        ide = None
        keys = elements.keys()
        progress = QProgressDialog(
            "Setting %s elements" % self.clName,
            "", 0, len(keys), self)
        progress.setWindowTitle("Set Elements")
        progress.setWindowModality(Qt.WindowModal)
        progress.setCancelButton(None)
        progress.show()
        for i in range(len(keys)):
            elname = keys[i]
            name = self.dashName(elname)
            dlg = self.createElement(name)
            try:
                if str(elements[elname]).strip():
                    dlg.set(elements[elname], new)
                else:
                    if hasattr(dlg, "createHeader"):
                        dlg.createHeader()
                    QMessageBox.warning(
                        self, "%s cannot be loaded" % self.clName,
                        "%s %s without content" % (self.clName, elname))
            except Exception:
                QMessageBox.warning(
                    self, "%s cannot be loaded" % self.clName,
                    "%s %s cannot be loaded" % (self.clName, elname))

            if hasattr(dlg, "dataSourceName"):
                dlg.dataSourceName = elname

            if hasattr(dlg, "addContextMenu"):
                dlg.addContextMenu(itemActions)

            if hasattr(dlg, "connectExternalActions"):
                actions = externalActions if externalActions else {}
                dlg.connectExternalActions(**actions)

            el = LabeledObject(name, dlg)
            if new:
                el.savedName = ""

            ide = id(el)
            self.elements[ide] = el
            if el.instance is not None:
                el.instance.id = el.id
                if new and hasattr(el.instance, "applied"):
                    el.instance.applied = True
            logger.info("setting %s" % name)
            progress.setValue(i)
        progress.setValue(len(keys))
        progress.close()
        return ide

    # replaces name special characters by underscore
    # \param name give name
    # \returns replaced element
    @classmethod
    def dashName(cls, name):
        return name

    # loads the element list from the given dictionary
    # \param externalActions dictionary with external actions
    # \param itemActions actions of the context menu
    def loadList(self, externalActions=None, itemActions=None):
        try:
            dirList = [
                l for l in os.listdir(self.directory)
                if (l.endswith(self.extention)
                    and (not self.disextention
                         or not l.endswith(self.disextention)))
            ]
        except Exception:
            try:
                if os.path.exists(os.path.join(os.getcwd(), self.name)):
                    self.directory = os.path.abspath(
                        os.path.join(os.getcwd(), self.name))
                else:
                    self.directory = os.getcwd()

                dirList = [
                    l for l in os.listdir(self.directory)
                    if (l.endswith(self.extention)
                        and (not self.disextention
                             or not l.endswith(self.disextention)))
                ]
            except Exception:
                return

        progress = QProgressDialog(
            "Loading %s elements" % self.clName,
            "", 0, len(dirList), self)
        progress.setWindowTitle("Load Elements")
        progress.setWindowModality(Qt.WindowModal)
        progress.setCancelButton(None)
        progress.forceShow()
        for i in range(len(dirList)):
            fname = dirList[i]
            name = self.nameFromFile(fname)
            dlg = self.createElement(name)
            dlg.load()
            if hasattr(dlg, "addContextMenu"):
                dlg.addContextMenu(itemActions)

            actions = externalActions if externalActions else {}
            if hasattr(dlg, "connectExternalActions"):
                dlg.connectExternalActions(**actions)

            el = LabeledObject(name, dlg)
            self.elements[id(el)] = el
            if el.instance is not None:
                el.instance.id = el.id
            logger.info("loading %s" % name)
            progress.setValue(i)
        progress.setValue(len(dirList))
        progress.close()


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # group form
    form = ElementList("../datasources")
#    form.elements={"title":"Test run 1", "run_cycle":"2012-1"}
    form.createGUI()
    form.show()
    app.exec_()

    if form.elements:
        logger.info("Other datasources:")
        for kk in form.elements.keys():
            logger.info("%s = '%s' " % (kk, form.elements[kk]))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DomTools.py
# Abstract Node dialog class

"""  DOM parser and tree view tools"""

from PyQt5.QtXml import QDomNode
import re


# abstract node dialog
class DomTools(object):

    # provides a list of elements from the given text
    # \param text give text
    # \param label element label
    # \returns list of element names from the given text
    @classmethod
    def findElements(self, text, label):
        variables = []
        index = text.find("$%s." % label)
        while index != -1:
            try:
                subc = re.finditer(
                    r"[\w]+",
                    text[(index + len(label) + 2):]
                ).next().group(0)
            except Exception:
                subc = ""
            name = subc.strip() if subc else ""
            if name:
                variables.append(name)
            index = text.find("$%s." % label, index + 1)
        return variables

    # provides row number of the given node
    # \param child child item
    # \param node parent node
    # \returns row number
    @classmethod
    def getNodeRow(cls, child, node):
        row = 0
        if node:
            children = node.childNodes()
            for i in range(children.count()):
                ch = children.item(i)
                if child == ch:
                    break
                row += 1
            if row < children.count():
                return row

    # provides node text for the given node
    # \param node DOM node
    # \returns string with node texts
    @classmethod
    def getText(cls, node):
        text = ""
        if node:
            child = node.firstChild()
            while not child.isNull():
                if child.nodeType() == QDomNode.TextNode:
                    text += child.toText().data()
                child = child.nextSibling()
        return text

    # provides row number of the given element
    # \param element DOM element
    # \param node DOM node
    # \returns row number
    @classmethod
    def __getElementRow(cls, element, node):
        row = 0
        if node:
            children = node.childNodes()
            for i in range(children.count()):
                ch = children.item(i)
                if ch.isElement() and element == ch.toElement():
                    break
                row += 1
            if row < children.count():
                return row

    # replaces node text for the given node
    # \param node parent DOM node
    # \param index of child text node
    # \param model Component model
    # \param text string with text
    @classmethod
    def replaceText(cls, node, index, model, text=None):
        if node:
            root = model.rootIndex.internalPointer().node
            children = node.childNodes()
            i = 0
            j = 0
            while i < children.count():
                child = children.item(i)
                if child.nodeType() == QDomNode.TextNode:
                    if j == 0 and text:
                        child.toText().setData(str(text))
                    else:
                        child.toText().setData(str(""))
                    j += 1
                i += 1

            if j == 0 and text:
                textNode = root.createTextNode(str(text))
                cls.appendNode(textNode, index, model)

    # removes node
    # \param node DOM node to remove
    # \param parent parent node index
    # \param model Component model
    @classmethod
    def removeNode(cls, node, parent, model):
        if not parent or not hasattr(parent, "internalPointer") \
                or not hasattr(parent.internalPointer(), "node"):
            return
        row = cls.getNodeRow(node, parent.internalPointer().node)
        if row is not None:
            model.removeItem(row, parent)

    # replaces node
    # \param oldNode old DOM node
    # \param newNode new DOM node
    # \param parent parent node index
    # \param model Component model
    @classmethod
    def replaceNode(cls, oldNode, newNode, parent, model):
        if not parent or not hasattr(parent, "internalPointer") \
                or not hasattr(parent.internalPointer(), "node"):
            return
        node = parent.internalPointer().node
        row = cls.getNodeRow(oldNode, node)
        if row is not None:
            model.removeItem(row, parent)
            if row < node.childNodes().count():
                model.insertItem(row, newNode, parent)
            else:
                model.appendItem(newNode, parent)

    # appends node
    # \param node DOM node to append
    # \param parent parent node index
    # \param model Component model
    @classmethod
    def appendNode(cls, node, parent, model):
        if model.appendItem(node, parent):
            return True

    # removes node element
    # \param element DOM node element to remove
    # \param parent parent node index
    # \param model Component model
    @classmethod
    def removeElement(cls, element, parent, model):
        if not parent or not hasattr(parent, "internalPointer") \
                or not hasattr(parent.internalPointer(), "node"):
            return
        row = cls.__getElementRow(element, parent.internalPointer().node)
        if row is not None:
            model.removeItem(row, parent)

    # replaces node element
    # \param oldElement old DOM node element
    # \param newElement new DOM node element
    # \param parent parent node index
    # \param model Component model
    @classmethod
    def replaceElement(cls, oldElement, newElement, parent, model):
        if not parent or not hasattr(parent, "internalPointer") \
                or not hasattr(parent.internalPointer(), "node"):
            return
        node = parent.internalPointer().node
        row = cls.__getElementRow(oldElement, node)
        if row is not None:
            model.removeItem(row, parent)
            if row < node.childNodes().count():
                model.insertItem(row, newElement, parent)
            else:
                model.appendItem(newElement, parent)

    # provides the first element in the tree with the given name
    # \param node DOM node
    # \param name child name
    # \returns DOM child node
    @classmethod
    def getFirstElement(cls, node, name):
        if node:
            child = node.firstChild()
            if child:
                while not child.isNull():
                    if child and child.nodeName() == name:
                        return child
                    child = child.nextSibling()

            child = node.firstChild()
            if child:
                while not child.isNull():
                    elem = cls.getFirstElement(child, name)
                    if elem:
                        return elem
                    child = child.nextSibling()


if __name__ == "__main__":
    pass
